/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : curses.c
 *  Author    : Dave Taylor
 *  Created   : ??-??-86
 *  Updated   : 14-10-91
 *  Release   : 2.1
 *  Notes     : This is a screen management library borrowed with permission
 *              from the Elm mail system (a great mailer--I highly recommend
 *              it!).This library was hacked to provide what tin needs.
 *  Copyright : Copyright (c) 1986 Dave Taylor
 *              The Elm Mail System  -  $Revision: 2.1 $   $State: Exp $
 */

#include <stdio.h>
#include <curses.h>
#include <sys/ioctl.h>

#ifdef TRUE
#undef TRUE
#define		TRUE		1
#endif

#ifdef FALSE
#undef FALSE
#define		FALSE		0
#endif

#define		BACKSPACE	'\b'
#define		VERY_LONG_STRING	2500

int LINES=23;
int COLS=80;

int inverse_okay = TRUE;

#ifdef BSD
#  ifndef BSD4_1
#    include <sgtty.h>
#  else
#    include <termio.h>
#  endif
#else
#  ifndef SYSV
#    include <termio.h>
#  endif
#endif

#include <ctype.h>

#include "tin.h"

#define TTYIN	0

#ifdef SHORTNAMES
# define _clearinverse	_clrinv
# define _cleartoeoln	_clrtoeoln
# define _cleartoeos	_clr2eos
#endif

#ifdef BSD
#define TCGETA	TIOCGETP
#define TCSETAW	TIOCSETP

struct sgttyb _raw_tty,
	      _original_tty;
#else
struct termio _raw_tty, 
              _original_tty;
#endif

static int _inraw = 0;                  /* are we IN rawmode?    */

#define DEFAULT_LINES_ON_TERMINAL	24
#define DEFAULT_COLUMNS_ON_TERMINAL	80

static char *_clearscreen, *_moveto, *_cleartoeoln, *_cleartoeos,
			*_setinverse, *_clearinverse, *_setunderline, *_clearunderline,
			*_terminalinit, *_terminalend;

static
int _lines,_columns;

static char _terminal[1024];              /* Storage for terminal entry */
static char _capabilities[1024];           /* String for cursor motion */

static char *ptr = _capabilities;	/* for buffering         */

int    outchar();			/* char output for tputs */
char  *tgetstr(),     		       /* Get termcap capability */
      *tgoto();				/* and the goto stuff    */

int InitScreen()
{
	int  tgetent();      /* get termcap entry */
	char termname[40], *p;
	char *strcpy(), *getenv();
	
	if ((p = getenv ("TERM")) == NULL) {
		fprintf (stderr,
		  "%s: TERM variable must be set to use screen capabilities\n",
		  progname);
		return (FALSE);
	}
	if (strcpy (termname, p) == NULL) {
		fprintf (stderr,"%s: Can't get TERM variable\n", progname);
		return (FALSE);
	}
	if (tgetent (_terminal, termname) != 1) {
		fprintf (stderr,"%s: Can't get entry for TERM\n", progname);
		return (FALSE);
	}

	/* load in all those pesky values */
	_clearscreen    = tgetstr ("cl", &ptr);
	_moveto         = tgetstr ("cm", &ptr);
	_cleartoeoln    = tgetstr ("ce", &ptr);
	_cleartoeos     = tgetstr ("cd", &ptr);
	_lines          = tgetnum ("li");
	_columns        = tgetnum ("co");
	_setinverse     = tgetstr ("so", &ptr);
	_clearinverse   = tgetstr ("se", &ptr);
	_setunderline   = tgetstr ("us", &ptr);
	_clearunderline = tgetstr ("ue", &ptr);
	_terminalinit   = tgetstr ("ti", &ptr);
	_terminalend    = tgetstr ("te", &ptr);

	if (_terminalinit) {
		tputs (_terminalinit, 1, outchar);
	}

	if (!_clearscreen) {
		fprintf (stderr,
			"%s: Terminal must have clearscreen (cl) capability\n",progname);
		return (FALSE);
	}
	if (!_moveto) {
		fprintf (stderr,
			"%s: Terminal must have cursor motion (cm)\n", progname);
		return (FALSE);
	}
	if (!_cleartoeoln) {
		fprintf (stderr,
			"%s: Terminal must have clear to end-of-line (ce)\n", progname);
		return (FALSE);
	}
	if (!_cleartoeos) {
		fprintf (stderr,
			"%s: terminal does not have clear to end-of-screen (ce). Simulating\n", progname);
/*
		fprintf (stderr,
			"%s: Terminal must have clear to end-of-screen (cd)\n", progname);
		return (FALSE);
*/		
	}
	if (_lines == -1)
		_lines = DEFAULT_LINES_ON_TERMINAL;
	if (_columns == -1)
		_columns = DEFAULT_COLUMNS_ON_TERMINAL;
	return (TRUE);
}

void ScreenSize(num_lines, num_columns)
	int *num_lines, *num_columns;
{
	/** returns the number of lines and columns on the display. **/

	if (_lines == 0) _lines = DEFAULT_LINES_ON_TERMINAL;
	if (_columns == 0) _columns = DEFAULT_COLUMNS_ON_TERMINAL;

	*num_lines = _lines - 1;		/* assume index from zero*/
	*num_columns = _columns;		/* assume index from one */
}

void EndWin ()
{
	if (_terminalend) {
		tputs (_terminalend, 1, outchar);
		fflush (stdout);
	}
}

void ClearScreen()
{
	/* clear the screen: returns -1 if not capable */

	tputs(_clearscreen, 1, outchar);
	fflush(stdout);      /* clear the output buffer */
}

void MoveCursor(row, col)
int row, col;
{
	/** move cursor to the specified row column on the screen.
            0,0 is the top left! **/

	char *stuff, *tgoto();

	stuff = tgoto(_moveto, col, row);
	tputs(stuff, 1, outchar);
	fflush(stdout);
}

void CleartoEOLN()
{
	/** clear to end of line **/

	tputs(_cleartoeoln, 1, outchar);
	fflush(stdout);  /* clear the output buffer */
}

void CleartoEOS()
{
	int i;
	
	/** clear to end of screen **/

	if (_cleartoeos) {
		tputs(_cleartoeos, 1, outchar);
	} else {
		for (i=_lines ; i < _lines ; i++) {
			MoveCursor (i, 0);
			CleartoEOLN ();
		}
	}
	fflush(stdout);  /* clear the output buffer */
}


void StartInverse()
{
	/** set inverse video mode **/

	if (_setinverse && inverse_okay)
		tputs(_setinverse, 1, outchar);
	fflush(stdout);
}


void EndInverse()
{
	/** compliment of startinverse **/

	if (_clearinverse && inverse_okay)
		tputs(_clearinverse, 1, outchar);
	fflush(stdout);
}


int RawState()
{
	/** returns either 1 or 0, for ON or OFF **/

	return( _inraw );
}


void Raw(state)
	int state;
{
	/** state is either TRUE or FALSE, as indicated by call **/

	if (state == FALSE && _inraw) {
	  (void) ioctl(TTYIN, TCSETAW, &_original_tty);
	  _inraw = 0;
	}
	else if (state == TRUE && ! _inraw) {

	  (void) ioctl(TTYIN, TCGETA, &_original_tty);	/** current setting **/

	  (void) ioctl(TTYIN, TCGETA, &_raw_tty);    /** again! **/
#ifdef BSD
	  _raw_tty.sg_flags &= ~(ECHO | CRMOD);	/* echo off */
	  _raw_tty.sg_flags |= CBREAK;	/* raw on    */
#else
	  _raw_tty.c_lflag &= ~(ICANON | ECHO);	/* noecho raw mode        */

	  _raw_tty.c_cc[VMIN] = '\01';	/* minimum # of chars to queue    */
	  _raw_tty.c_cc[VTIME] = '\0';	/* minimum time to wait for input */
#endif

	  (void) ioctl(TTYIN, TCSETAW, &_raw_tty);

	  _inraw = 1;
	}
}

int ReadCh()
{
	/** read a character with Raw mode set! **/

	register int result;
	char ch;
	
#ifdef READ_CHAR_HACK
#undef getc
	ch = getc (stdin);
	return ((ch == EOF) ? EOF : ch & 0xFF);
#else
	result = read(0, &ch, 1);
        return((result <= 0 ) ? EOF : ch & 0xFF);
#endif		
}


int outchar(c)
char c;
{
	/** output the given character.  From tputs... **/
	/** Note: this CANNOT be a macro!              **/

	putc(c, stdout);
}
