/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : main.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 04-10-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

char *version = "v1.0";
char cvers[LEN];
char nntp_server[LEN];

int NOTESLINES;						/* set in set_win_size () */
int RIGHT_POS;						/* set in set_win_size () */
int MORE_POS;						/* set in set_win_size () */
int max_subj = 0;
int max_from = 0;
int max_active;
int group_hash[TABLE_SIZE];			/* group name --> active[] */
int *my_group;						/* .newsrc --> active[] */
int *unread;						/* highest art read in group */
int num_active;						/* one past top of active */
int local_top;						/* one past top of my_group */
int catchup = FALSE;				/* mark all arts read in all subscribed groups */
int update = FALSE;					/* update index files only mode */
int update_fork = FALSE;			/* update index files by forked tin -u */
int verbose = FALSE;				/* update index files only mode */
int start_line_offset;				/* used by invoke_editor for line no. */
int read_news_via_nntp = FALSE;		/* read news locally or via NNTP */
int max_art;
int top = 0;
int top_base;
int tin_uid;
int tin_gid;
int real_uid;
int real_gid;
int local_index;				/* do private indexing? */

long *base;

struct group_ent *active;			/* active file */
struct header *arts;


void main (argc, argv)
	int argc;	
	char *argv[];
{
	extern int optind;
	extern char *optarg;
	int c, i, created;
	int check_any_unread = FALSE;
	int notify_new_groups = FALSE;
	int start_any_unread = FALSE;
	int start_groupnum = 0;
	struct stat st;
	
	debug = FALSE;

	set_signal_handlers ();		/* all except SIGINT ctrl-C */

	basename (argv[0], progname);

	if (progname[0] == 'r') {	/* rtin so read news remotely via NNTP */
#ifdef USE_NNTP			
		read_news_via_nntp = TRUE;
#else
		error_message ("Reading news via NNTP is not enabled. Recompile %s with -DUSE_NNTP.\n", progname);
		exit (1);
#endif
	}

	tin_uid = geteuid ();
	tin_gid = getegid ();
	real_uid = getuid ();
	real_gid = getgid ();

	/*
	 * we're setuid, so index in /usr/spool/news unless user root
	 */
	if (tin_uid != real_uid && real_uid != 0) {
		local_index = FALSE;
	} else {	/* index in users home directory ~/.tin/.index */
		local_index = TRUE;
	}

	init_selfinfo ();	/* set up char *'s: homedir, newsrc, etc. */
	init_alloc ();		/* allocate initial array sizes */

	while ((c = getopt (argc, argv, "cd:Df:hm:M:np:rs:SuUvzZ")) != EOF) {
		switch(c) {
			case 'c':
				catchup = TRUE;
				update = TRUE;
				break;
				
			case 'd':
				my_strncpy (savedir, optarg, LEN);
				set_real_uid_gid ();
				if (stat (savedir, &st) == -1) {
					mkdir (savedir, 0755);
				}
				set_tin_uid_gid ();
				break;

			case 'D':		/* debug mode */
				debug = TRUE;
				break;

			case 'f':
				my_strncpy (newsrc, optarg, LEN);
				break;

			case 'm':
				my_strncpy (mailer, optarg, LEN);
				break;

			case 'M':
				my_strncpy (maildir, optarg, LEN);
				set_real_uid_gid ();
				if (stat (maildir, &st) == -1) {
					mkdir (maildir, 0755);
				}
				set_tin_uid_gid ();
				break;

			case 'n':
				notify_new_groups = TRUE;
				break;

			case 'p':
				my_strncpy (printer, optarg, LEN);
				default_printer = FALSE;
				break;

			case 'r':	/* read news remotely from default NNTP server */
#ifdef USE_NNTP			
				read_news_via_nntp = TRUE;
#else
				printf ("Option not enabled. Recompile %s with -DUSE_NNTP.\n", progname);
				exit (1);
#endif
				break;

			case 's':
				my_strncpy (spooldir, optarg, LEN);
				break;

			case 'S':	/* save new news to dir structure */
				save_news = TRUE;
				update = TRUE;
				break;

			case 'u':
				update = TRUE;
				break;

			case 'U':
				update_fork = TRUE;
				update = TRUE;
				break;

			case 'v':
				verbose = TRUE;
				break;

			case 'z':
				start_any_unread = TRUE;
				update = TRUE;
				break;

			case 'Z':
				check_any_unread = TRUE;
				update = TRUE;
				break;

			case 'h':
			case '?':
			default:
				usage (progname);
				exit (1);
		}
	}

	sprintf (page_header, "%s %s PL%d%s", progname, version, PATCHLEVEL, is_remote ()); 	
	sprintf (cvers, "%s (c) Copyright 1991 Iain Lea.", page_header);

	if (! update) {
		printf ("%s\n", cvers);
		fflush (stdout);
	}

	hash_init ();
	for (i = 0; i < TABLE_SIZE; i++) {
		group_hash[i] = -1;
	}

	nntp_startup ();				/* connect server if we are using nntp */
	created = read_active ();		/* load the active file into active[] */

	if (optind < argc) {
		while (optind < argc) {
			if (add_group (argv[optind], TRUE) < 0) {
				error_message (txt_not_in_active_file, argv[optind]);
			}
			optind++;
		}
	} else {
		backup_newsrc ();
		read_newsrc (TRUE);
		mark_unthreaded_groups ();
	}

	if (kill_articles) {		/* read in users kill file */
		read_kill_file ();
	}
	
	/*
	 *  check/start if any new/unread articles
	 */
	if (check_any_unread) {
		i = check_start_save_any_news (CHECK_ANY_NEWS);
		exit (i);
	} else if (start_any_unread) {
		start_groupnum = check_start_save_any_news (START_ANY_NEWS);
		if (start_groupnum == -1) {		/* no new/unread news so exit */
			exit (0);
		}
		update = FALSE;
	}

	/*
	 *  save any new articles to savedir structure for later reading
	 */
	if (save_news) {
		i = catchup;			/* set catchup to FALSE */
		catchup = FALSE;
		do_update ();
		catchup = i;			/* set catchup to previous value */
		check_start_save_any_news (SAVE_ANY_NEWS);
		exit (0);
	}
	
	/*
	 *  update index files
	 */
	if (update || update_fork) {
		COLS = DEFAULT_COLS;			/* set because curses has not started */ 
		if (update_fork) {
			catchup = FALSE;			/* turn off msgs when running forked */ 
			verbose = FALSE;
			if (fork () == 0) {			/* fork child to update indexes in background */
				nntp_startup ();		/* connect server if we are using nntp */
				thread_arts = FALSE;	/* stop threading to run faster */
				do_update ();
				nntp_finish ();			/* connect server if we are using nntp */
				exit (0);
			}
			update = FALSE;
		} else {
			thread_arts = FALSE;	/* stop threading to run faster */
			do_update ();
			exit (0);
		}
	}

	if (! InitScreen ()) {
		error_message (txt_screen_init_failed, progname);
		exit (1);
	}

	ScreenSize (&LINES, &COLS);		/* screen size from termcap entry */
	Raw (TRUE);

	set_win_size (&LINES, &COLS);	/* screen size doing ioctl() call */

	if (notify_new_groups && ! created) {	/* check for new groups */
		notify_groups ();
	}
	 
#ifdef READ_CHAR_HACK
	setbuf (stdin, 0);
#endif

	selection_index (start_groupnum);

	tin_done (0);
}

/*
 * usage
 */

void usage (progname)
	char *progname;
{
	sprintf (msg, "%s %s PL%d - threaded usenet newsreader. Copyright 1991 Iain Lea.\n", progname, version, PATCHLEVEL);
	error_message (msg, (char *) 0);
	error_message ("Usage: %s [options] [newsgroups]", progname);
	error_message ("  -c       mark all articles as read in subscribed newsgroups", (char *) 0);
	error_message ("  -d dir   save articles to directory [default=%s]", savedir);
	error_message ("  -f file  subscribed newsgroups file [default=%s]", newsrc);
	error_message ("  -h       help", (char *) 0);
	error_message ("  -m file  mail program [default=%s]", DEFAULT_MAILER);
	error_message ("  -n       notify user of any new newsgroups since last session", (char *) 0);
	error_message ("  -M dir   mailbox directory [default=%s]", maildir);
	error_message ("  -p file  print program with options [default=%s]", DEFAULT_PRINTER);
	error_message ("  -r       read news remotely from default NNTP server", DEFAULT_PRINTER);
	error_message ("  -s dir   spool directory [default=%s]", SPOOLDIR);
	error_message ("  -S       save all new articles for later reading", (char *) 0);
	error_message ("  -u       update index files only", (char *) 0);
	error_message ("  -U       fork %s -u to update index files in background", progname);
	error_message ("  -v       update index files only (verbose)", (char *) 0);
	error_message ("  -z       start if any unread news", (char *) 0);
	error_message ("  -Z       return status indicating if any unread news", (char *) 0);
	error_message ("\nMail bug reports/gripes/comments to %s", BUG_REPORT_ADDRESS);
}
