/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : misc.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 15-10-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"


char *mailbox_name = (char *) 0;
int  mailbox_size;

char rcdir[LEN];
char rcfile[LEN];
char indexdir[LEN];
char killfile[LEN];
char postfile[LEN];
char unthreadfile[LEN];

char active_file[LEN];
char homedir[LEN];
char userid[LEN];
char delgroups[LEN];
char newsrc[LEN];
char newnewsrc[LEN];
char add_addr[LEN];			/* address to add to rR reply to author with mail */
char bug_addr[LEN];			/* address to add send bug reports to */
char txt_help_bug_report[LEN];	/* address to add send bug reports to */
char reply_to[LEN];			/* reply-to address */
char my_org[LEN];			/* organization */
char sig[LEN];
char signature[LEN];
char default_shell_command[LEN];	/* offers user default choice */
char killsubj[LEN];			/* contains Subject:'s not to be shown */
char killfrom[LEN];			/* conatins From:'s not to be shown */
char page_header[LEN];		/* page header of pgm name and version */
char savedir[LEN];			/* directory to save articles to */
char spooldir[LEN];			/* directory where new is */
char mailer[LEN];			/* mail program */
char maildir[LEN];			/* mailbox dir where = saves are stored */
char mailbox[LEN];			/* system mailbox for each user */
char printer[LEN];			/* printer program */
char article[LEN];			/* ~/.article file */
char dead_article[LEN];		/* ~/dead.article file */
char progname[LEN];			/* program name */
int sort_art_type;			/* sort arts[] array by subject,from or date field */
int save_separate;			/* save a each part of a thread to a separate file */
int save_archive_name;		/* save thread with name from Archive-name: field */
int print_header;			/* print all of mail header or just Subject: & From lines */
int pos_first_unread;		/* position cursor at first/last unread article */
int show_author;			/* show Subject & From or only Subject in group menu */
int draw_arrow_mark;		/* draw -> or highlighted bar */
int kill_articles;			/* kill articles matching kill file contents */
int mark_saved_read;		/* mark saved article/thread as read */
int post_proc_type;			/* type of post processing to be performed */
int default_printer;		/* set to false if user give a printer with -p switch */
int save_news;				/* save all arts to savedir structure */
int thread_arts;			/* thread/unthread articles for viewing */
int num_of_tagged_files;

/*
 * init_selfinfo
 *   Deterimines users home directory, userid, and a path
 *   for an rc file in the home directory
 */

void init_selfinfo ()
{
	extern char *getenv ();
	extern char *getlogin ();
	extern struct passwd *getpwuid ();
	extern struct passwd *getpwnam ();
	char nam[LEN];
	char *p;
	FILE *fp;
	struct passwd *myentry;
	struct stat sb;

	if ((p = getlogin ()) != (char *) 0) {
		myentry = getpwnam (p);
	} else {
		myentry = getpwuid (getuid ());
	}

	strcpy (userid, myentry->pw_name);

	if ((p = getenv("HOME")) != NULL) {
		strcpy (homedir, p);
	} else {
		strcpy (homedir, myentry->pw_dir);
	}

#ifdef USE_INVERSE_HACK
	draw_arrow_mark = TRUE;
#else
	draw_arrow_mark = FALSE;
#endif
#ifdef BSD
	default_printer = TRUE;
#else	
	default_printer = FALSE;
#endif
	kill_articles = FALSE;
	mark_saved_read = TRUE;
	num_of_tagged_files = 0;
	post_proc_type = POST_PROC_NONE;
	pos_first_unread = TRUE;
	print_header = FALSE;
	save_separate = TRUE;
	save_archive_name = TRUE;
	show_author = TRUE;
	sort_art_type = SORT_BY_NOTHING;
	save_news = FALSE;
	thread_arts = TRUE;
	
	killsubj[0] = '\0';
	killfrom[0] = '\0';
	
	nntp_server[0] = '\0';
	author_search_string[0] = '\0';
	group_search_string[0] = '\0';
	subject_search_string[0] = '\0';
	art_search_string[0] = '\0';
	default_mail_address[0] = '\0';
	default_regex_pattern[0] = '\0';
	default_save_file[0] = '\0';
	default_shell_command[0] = '\0';
	default_post_subject[0] = '\0';
	
	sprintf (rcdir,  "%s/%s", get_val ("TINDIR", homedir), RCDIR);
	sprintf (indexdir, "%s/%s", rcdir, INDEXDIR);
	sprintf (rcfile,   "%s/%s", rcdir, RCFILE);
	sprintf (killfile, "%s/%s", rcdir, KILLFILE);
	sprintf (postfile, "%s/%s", rcdir, POSTFILE);
	sprintf (unthreadfile, "%s/%s", rcdir, UNTHREADFILE);

	sprintf (signature, "%s/.signature", homedir);
	sprintf (sig, "%s/.Sig", homedir);
	sprintf (newsrc, "%s/.newsrc", homedir);
	sprintf (newnewsrc, "%s/.newnewsrc", homedir);
	sprintf (delgroups, "%s/.delgroups", homedir);
	sprintf (active_file, "%s/active", LIBDIR);
	sprintf (savedir, "%s/News", homedir);
	sprintf (maildir, "%s/%s", homedir, DEFAULT_MAILDIR);
	sprintf (mailbox, "%s/%s", DEFAULT_MAILBOX, userid);
	sprintf (dead_article, "%s/dead.article", homedir);
	sprintf (article, "%s/.article", homedir);

	strcpy (mailer, DEFAULT_MAILER);
	strcpy (printer, DEFAULT_PRINTER);
	strcpy (spooldir, SPOOLDIR);
	strcpy (bug_addr, BUG_REPORT_ADDRESS);

	set_real_uid_gid ();
	
	if (stat (rcdir, &sb) == -1) {
		mkdir (rcdir, 0755);
	}
	if (stat (indexdir, &sb) == -1) {
		mkdir (indexdir, 0755);
	}
	if (stat (postfile, &sb) == -1) {
		if ((fp = fopen (postfile, "w")) != NULL) {
			fclose (fp);
		}
	}
	if (stat (unthreadfile, &sb) == -1) {
		if ((fp = fopen (unthreadfile, "w")) != NULL) {
			fclose (fp);
		}
	}

	read_rcfile ();

	if (stat (savedir, &sb) == -1) {
		mkdir (savedir, 0755);
	}
	if (stat (maildir, &sb) == -1) {
		mkdir (maildir, 0755);
	}
	if (stat (active_file, &sb) >= 0)
		goto got_active;

/*
 *  I hate forgetting to define LIBDIR correctly.  Guess a
 *  couple of likely places if it's not where LIBDIR says it is.
 */

	strcpy (active_file, "/usr/lib/news/active");
	if (stat (active_file, &sb) >= 0)
		goto got_active;

	strcpy (active_file, "/usr/local/lib/news/active");
	if (stat (active_file, &sb) >= 0)
		goto got_active;

	strcpy (active_file, "/usr/public/lib/news/active");
	if (stat (active_file, &sb) >= 0)
		goto got_active;

/*
 *  Oh well. Revert to what LIBDIR says it is to produce a
 *  useful error message when read_active () fails later.
 */

	sprintf (active_file, "%s/active", LIBDIR);

got_active:

/*
 *  check enviroment for organization
 */
	my_org[0] = '\0';
	if ((p = getenv ("ORGANIZATION")) != NULL) {
		my_strncpy (my_org, p, LEN);
		goto got_org;
	}

/*
 *  check ~/.org for organization
 */
	sprintf (nam, "%s/organization", rcdir);
	fp = fopen (nam, "r");

/*
 *  check LIBDIR/organization for system wide organization
 */
	if (fp == NULL) {
		sprintf (nam, "%s/organization", LIBDIR);
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/usr/lib/news/organization");
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/usr/local/lib/news/organization");
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/usr/public/lib/news/organization");
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/etc/organization");
		fp = fopen (nam, "r");
	}

	if (fp != NULL) {
		if (fgets (my_org, sizeof (my_org), fp) != NULL) {
			for (p = my_org; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_org:;

/*
 *  check enviroment for REPLYTO
 */
	reply_to[0] = '\0';
	if ((p = getenv ("REPLYTO")) != NULL) {
		my_strncpy (reply_to, p, LEN);
		goto got_reply;
	}

	sprintf (nam, "%s/reply_to", rcdir);
	if ((fp = fopen (nam, "r")) != NULL) {
		if (fgets (reply_to, sizeof (reply_to), fp) != NULL) {
			for (p = reply_to; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_reply:;

/*
 *  check enviroment for ADD_ADDRESS
 */
	add_addr[0] = '\0';
	if ((p = getenv ("ADD_ADDRESS")) != NULL) {
		my_strncpy (add_addr, p, LEN);
		goto got_add_addr;
	}

	sprintf (nam, "%s/add_address", rcdir);
	if ((fp = fopen (nam, "r")) != NULL) {
		if (fgets (add_addr, sizeof (add_addr), fp) != NULL) {
			for (p = add_addr; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_add_addr:;

/*
 *  check enviroment for BUG_ADDRESS
 */
	if ((p = getenv ("BUG_ADDRESS")) != NULL) {
		my_strncpy (bug_addr, p, LEN);
		goto got_bug_addr;
	}

	sprintf (nam, "%s/bug_address", rcdir);
	if ((fp = fopen (nam, "r")) != NULL) {
		if (fgets (bug_addr, sizeof (bug_addr), fp) != NULL) {
			for (p = bug_addr; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_bug_addr:;
	sprintf (txt_help_bug_report, txt_help_bug, bug_addr);

	set_tin_uid_gid ();
}


/*
 *  Which base note (an index into base[]) does a respnum
 *  (an index into arts[]) corresponsd to?
 *
 *  In other words, base[] points to an entry in arts[] which is
 *  the head of a thread, linked with arts[].thread.  For any q: arts[q],
 *  find i such that base[i]->arts[n]->arts[o]->...->arts[q]
 */

int which_base (n)
	int n;
{
	register int i, j;

	for (i = 0; i < top_base; i++) {
		for (j = (int) base[i] ; j >= 0 ; j = arts[j].thread) {
			if (j == n) {
				return i;
			}
		}
	}

	error_message (txt_cannot_find_base_art, (char *) atoi (n));
	return -1;
}

/*
 *  Find how deep in a thread a response is.  Start counting at zero
 */

int which_resp (n)
	int n;
{
	int i, j;
	int num = 0;

	i = which_base (n);

	for (j = (int) base[i]; j != -1; j = arts[j].thread)
		if (j == n)
			break;
		else
			num++;

	return num;
}

/*
 *  Given an index into base[], find the number of responses for
 *  that basenote
 */

int nresp (n)
	int n;
{
	int i;
	int oldi = -3;
	int sum = 0;

	assert (n < top_base);

	for (i = (int) base[n]; i != -1; i = arts[i].thread) {
		assert (i != -2);
		assert (i != oldi);
		oldi = i;
		sum++;
	}

	return sum - 1;
}


void asfail(file, line, cond)
	char	*file;
	int	line;
	char	*cond;
{
	fprintf(stderr, "%s: assertion failure: %s (%d): %s\n",progname,file,line,cond);
	fflush (stderr);
	exit(1);
}


void copy_fp(a, b, prefix)
	FILE *a;
	FILE *b;
	char *prefix;
{
	char buf[8192];

	while (fgets (buf, sizeof (buf), a) != NULL)
		fprintf (b, "%s%s", prefix, buf);
}


char *get_val(env, def)
	char *env;		/* Environment variable we're looking for	*/
	char *def;		/* Default value if no environ value found	*/
{
	char *ptr;

	if ((ptr = (char *) getenv(env)) != NULL)
		return (ptr);
	else
		return (def);
}


int invoke_editor (nam)
	char *nam;
{
	char buf[LEN];
	static char editor[LEN];
	static int first = TRUE;

	if (first) {
		char *my_editor = getenv ("VISUAL");

		strcpy (editor, my_editor != NULL ? my_editor : get_val ("EDITOR", DEFAULT_EDITOR));
		first = FALSE;
	}

#ifdef DONT_USE_START_LINE
	sprintf (buf, "%s %s", editor, nam);
#else
	sprintf (buf, "%s +%d %s", editor, start_line_offset, nam);
#endif

	printf ("%s", buf);
	return invoke_cmd (buf);
}


void shell_escape ()
{
	char shell[LEN];
	char *p;

#ifdef SIGTSTP
	void (*susp)();
#endif

	sprintf (msg, txt_shell_escape, default_shell_command);
	
	if (! parse_string (msg, shell))
		strcpy (shell, get_val ("SHELL", DEFAULT_SHELL));

	for (p = shell; *p && (*p == ' ' || *p == '\t'); p++)
		continue;

	if (*p) {
		my_strncpy (default_shell_command, p, LEN);
	} else {
		if (default_shell_command[0]) {
			my_strncpy (shell, default_shell_command, LEN);
		} else {
			strcpy (shell, get_val ("SHELL", DEFAULT_SHELL));
		}
		p = shell;
	}

	ClearScreen ();
	sprintf (msg, "Shell Command (%s)", p);
	center_line (0, TRUE, msg);
	MoveCursor (INDEX_TOP, 0);
	
	Raw (FALSE);

	set_real_uid_gid ();

#ifdef SIGTSTP
	if (do_sigtstp)
		susp = signal (SIGTSTP, SIG_DFL);
#endif

	system (p);

#ifdef SIGTSTP
	if (do_sigtstp)
		signal (SIGTSTP, susp);
#endif

	set_tin_uid_gid ();

	Raw (TRUE);

	mail_setup ();

	continue_prompt ();

	if (draw_arrow_mark) {
		ClearScreen ();
	}
}


void tin_done (ret)
	int ret;
{
	char group_path[LEN], *p;
	int ask = TRUE;
	register int i, j;
	
	/*
	 * check if any groups were read & ask if they should marked read
	 */
	for (i = 0 ; i < local_top ; i++) {
		if (active[my_group[i]].read) {
			if (ask) {
				if (prompt_yn (LINES, "Catchup all groups entered during this session? (y/n): ", 'n')) {
					ask = FALSE;
					thread_arts = FALSE;	/* speeds up index loading */
				} else {
					break;
				}
			}
			sprintf (msg, "Catchup %s...", active[my_group[i]].name);
			wait_message (msg);
			strcpy (group_path, active[my_group[i]].name);
			for (p = group_path; *p; p++) {
				if (*p == '.') {
					*p = '/';
				}
			}
			index_group (active[my_group[i]].name, group_path);
			for (j = 0; j < top; j++) {
				arts[j].unread = ART_READ;
			}
			update_newsrc (active[my_group[i]].name, my_group[i], FALSE);
		}
	}
	nntp_finish ();			/* disconnect from NNTP server */
	free_all_arrays ();		/* deallocate all arrays */
	ClearScreen ();
	EndWin ();
	Raw (FALSE);
	exit (ret);
}

/*
 *  Load the active file into active[] and create copy of active ~/.tin/active
 */

int read_active ()
{
	FILE *fp;
	char *p, *q;
	char buf[LEN];
	int created, i;
	long h;
	
	num_active = 0;

	if ((fp = open_active_fp ()) == NULL) {
		if (compiled_with_nntp) {
			printf ("Cannot open %s. Try %s -r to read news via NNTP.\n",
				active_file, progname);
		} else {
			printf (txt_cannot_open, active_file);
			putchar ('\n');
		}
		fflush (stdout);
		exit (1);
	}

	while (fgets (buf, sizeof (buf), fp) != NULL) {
		for (p = buf; *p && *p != ' '; p++)
			continue;
		if (*p != ' ') {
			error_message (txt_bad_active_file, "");
			continue;
		}
		*p++ = '\0';

		if (num_active >= max_active)
			expand_active ();

		h = hash_groupname (buf);

		if (group_hash[h] == -1) {
			group_hash[h] = num_active;
		} else {				/* hash linked list chaining */
			for (i=group_hash[h]; active[i].next >= 0; i=active[i].next) {
				if (strcmp(active[i].name, buf) == 0) {
					goto read_active_continue;		/* kill dups */
				}
			}
			if (strcmp(active[i].name, buf) == 0)
				goto read_active_continue;
			active[i].next = num_active;
		}

		for (q = p; *q && *q != ' '; q++)
			continue;
		if (*q != ' ') {
			error_message (txt_bad_active_file, "");
			continue;
		}

		active[num_active].name = str_dup (buf);
		active[num_active].max = (long) atol (p);
		active[num_active].min = (long) atol (q);
		active[num_active].next = -1;		/* hash chaining */
		active[num_active].flag = NOTGOT;	/* not in my_group[] yet */
		active[num_active].read = FALSE;	/* read/unread */
		active[num_active].thread = TRUE;	/* thread articles */

		num_active++;

read_active_continue:;

	}
	fclose (fp);

	/*
	 *  exit if active file is empty
	 */
	if (! num_active) {
		error_message ("%s contains no newsgroups. Exiting.", active_file);
		exit (1);
	}

	/*
	 *  create backup of LIBDIR/active for use by -n option to notify new groups 
	 */
	created = backup_active (TRUE);

	debug_print_active ();

	return (created);
}

/*
 *  Mark any groups in my_group[] that are in ~/.tin/unthread so they
 *  will not be threaded
 */
 
void mark_unthreaded_groups ()
{
	FILE *fp;
	char buf[LEN];
	int i;
	long h;
	
	if ((fp = fopen (unthreadfile, "r")) == NULL) {
		error_message (txt_cannot_open, unthreadfile);
		return;
	}

	while (fgets (buf, sizeof (buf), fp) != NULL) {
		buf[strlen (buf)-1] = '\0'; 
		h = hash_groupname (buf);

		if (debug) {
			printf ("Unthreading %s...\n", buf);
		}

		if ((i = group_hash[h]) >= 0) {
			active[i].thread = FALSE;
		} else {
			for (i=group_hash[h]; active[i].next >= 0; i=active[i].next) {
				if (strcmp (active[i].name, buf) == 0) {
					active[i].thread = FALSE;
					break;
				}
			}
		}
	}
	fclose (fp);
}

/*
 *  create ~/.tin/active from LIBDIR/active if it does not exist 
 */

int backup_active (create)
	int create;
{
	char buf[LEN];
	FILE *fp;
	int created = FALSE;
	int i;
	struct stat sb;
	
	set_real_uid_gid ();
	
	sprintf (buf, "%s/active", rcdir);
	
	if (create) {
		if (stat (buf, &sb) != -1) {
			goto backup_active_done;
		}
	}
	
	if ((fp = fopen (buf, "w")) != NULL) {
		for (i = 0; i < num_active ; i++) {	/* for each group */
			fprintf (fp, "%s\n", active[i].name);
		}
		fclose (fp);
		chmod (buf, 0644);
		created = TRUE;
	}

backup_active_done:
	set_tin_uid_gid ();
	return (created);
}


void add_signature (fp, flag)
	FILE *fp;
	int flag;
{
	FILE *sigf;

	if ((sigf = fopen(signature, "r")) != NULL) {
		if (flag) {
			fprintf(fp, "\n--\n");
			copy_fp(sigf, fp, "");
		}
		fclose(sigf);
		return;
	}

	if ((sigf = fopen(sig, "r")) != NULL) {
		fprintf(fp, "\n--\n");
		copy_fp(sigf, fp, "");
		fclose(sigf);
	}
}


#ifdef USE_MKDIR
mkdir (path, mode)
	char *path;
	int mode;
{
	char buf[LEN];
	struct stat sb;

	sprintf(buf, "mkdir %s", path);
	if (stat (path, &sb) == -1) {
		system (buf);
		chmod (path, mode);
	}
}
#endif


long hash_groupname (buf)		/* hash group name for fast lookup later */
	char *buf;
{
	unsigned long h;
	unsigned char *t = (unsigned char *) buf;

	h = *t++;
	while (*t)
		h = ((h << 1) ^ *t++) % TABLE_SIZE;

	return h;
}


void rename_file (old_filename, new_filename)
	char *old_filename;
	char *new_filename;
{	
	char buf[LEN];
	
	unlink (new_filename);
	if (link (old_filename, new_filename) == 1) {
		sprintf (buf, txt_rename_error, old_filename, new_filename);
		error_message (buf, "TWO");
		return;
	}
	if (unlink (old_filename) == -1) {
		sprintf (buf, txt_rename_error, old_filename, new_filename);
		error_message (buf, "THREE");
		return;
	}
}


char *str_dup (str)
	char *str;
{
	char *dup = (char *) 0;

	assert (str != (char *) 0);
	
	if (str) {
		dup = my_malloc (strlen (str)+1);
		strcpy (dup, str);
	}
	return dup;
}


int invoke_cmd (nam)
	char *nam;
{
	int ret;

#ifdef SIGTSTP
	void (*susp)();
#endif

	Raw (FALSE);
	set_real_uid_gid ();

#ifdef SIGTSTP
	if (do_sigtstp)
		susp = signal(SIGTSTP, SIG_DFL);
#endif

	ret = system (nam);

#ifdef SIGTSTP
	if (do_sigtstp)
		signal (SIGTSTP, susp);
#endif

	set_tin_uid_gid ();
	Raw (TRUE);

	return ret == 0;
}


void draw_percent_mark (cur_num, max_num)
	int cur_num;
	int max_num;
{
	char buf[32];
	int percent = 0;

	if (NOTESLINES <= 0) {
		return;
	}

	if (cur_num <= 0 && max_num <= 0) {
		return;
	}
		
	percent = cur_num * 100 / max_num;
	sprintf (buf, "%s(%d%%) [%d/%d]", txt_more, percent, cur_num, max_num);
	MoveCursor (LINES, (COLS - (int) strlen (buf))-(1+BLANK_PAGE_COLS));
	StartInverse ();	
	printf ("%s", buf);
	fflush (stdout);
	EndInverse ();
}


void set_real_uid_gid ()
{
#if defined(BSD) && ! defined(SINIX)
	if (seteuid (real_uid) == -1) {
		error_message ("Error seteuid(real) failed", (char *) 0);
	}
	if (setegid (real_gid) == -1) {
		error_message ("Error setegid(real) failed", (char *) 0);
	}
#else
	if (setuid (real_uid) == -1) {
		error_message ("Error setuid(real) failed", (char *) 0);
	}
	if (setgid (real_gid) == -1) {
		error_message ("Error setgid(real) failed", (char *) 0);
	}
#endif
}


void set_tin_uid_gid ()
{
#if defined(BSD) && ! defined(SINIX)
	if (seteuid (tin_uid) == -1) {
		error_message ("Error seteuid(tin) failed", (char *) 0);
	}
	if (setegid (tin_gid) == -1) {
		error_message ("Error setegid(tin) failed", (char *) 0);
	}
#else
	if (setuid (tin_uid) == -1) {
		error_message ("Error setuid(tin) failed", (char *) 0);
	}
	if (setgid (tin_gid) == -1) {
		error_message ("Error setgid(tin) failed", (char *) 0);
	}
#endif
}


void notify_groups ()
{
	char buf[LEN];
	FILE *fp;
	int group_not_found;
	int i, j, index;
	int num = 0;
	int update_old_active = FALSE;
	struct notify_t {
		char name[LEN];
		int len;
	} *old_active;
	
	set_real_uid_gid ();
	
	sprintf (buf, "%s/active", rcdir);
	
	if ((fp = fopen (buf, "r")) == NULL) {
		error_message (txt_cannot_open, buf);
		goto notify_groups_done;
	}

	Raw (FALSE);
	
	printf ("Checking active list for new newsgroups...");
	fflush (stdout);
	
	old_active = (struct notify_t *) my_malloc ((unsigned) sizeof (struct notify_t) * num_active);
	if (old_active == (struct notify_t *) 0) {
		error_message (txt_out_of_memory, progname);
		goto notify_groups_done;
	}
	
	while (fgets (old_active[num].name, sizeof (old_active[num].name), fp) != NULL) {
		old_active[num].len = strlen (old_active[num].name)-1;
		old_active[num].name[old_active[num].len] = '\0';
		num++;
		if (num == num_active) {
			break;
		}
	}

	for (i = 0 ; i < num_active ; i++) {	
		group_not_found = TRUE;
		for (j=0; j < num ; j++) {
			if (strncmp (old_active[j].name, active[i].name, old_active[j].len) == 0) {
				group_not_found = FALSE;		/* found it so read in next group */
				break;
			}
		}

		if (group_not_found) {
/*
&& strlen (active[i].name) == old_active[j].len) {
*/
			update_old_active = TRUE;
			printf ("\r\nSubscribe to %s (y/n): ", active[i].name);
			fflush (stdout);
			scanf ("%s", buf);
			if (buf[0] == 'y') {
				index = add_group (active[i].name, TRUE);
				subscribe (active[my_group[index]].name, ':',
					my_group[index], FALSE);
			}
			printf ("Checking...");
			fflush (stdout);
		}
	}
	fclose (fp);

	if (old_active != (struct notify_t *) 0) {
		free (old_active);
		old_active = (struct notify_t *) 0;
	}

	Raw (TRUE);

	/*
	 *  write active[] to ~/.tin/active
	 */
	if (update_old_active) {
		backup_active (FALSE);
	}

notify_groups_done:
	set_tin_uid_gid ();
}


void basename (dirname, program)
	char *dirname;		/* argv[0] */
	char *program;		/* progname is returned */
{
	int i;
	
	strcpy (program, dirname);
	
	for (i=(int) strlen (dirname)-1 ; i ; i--) {
		if (dirname[i] == '/') {
			strcpy (program, dirname+(i+1));
			break;
		}
	}
}


/*
 *  Record size of mailbox so we can detect if new mail has arrived
 */

void mail_setup ()
{
	struct stat buf;

	mailbox_name = get_val ("MAIL", mailbox);

	if (stat (mailbox_name, &buf) >= 0) {
		mailbox_size = buf.st_size;
	} else {
		mailbox_size = 0;
	}
}

/*
 *  Return TRUE if new mail has arrived
 */

int mail_check ()
{
	struct stat buf;

	if (mailbox_name != NULL
	&&  stat(mailbox_name, &buf) >= 0
	&&  mailbox_size < buf.st_size)
		return TRUE;

	return FALSE;
}
