/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : page.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 10-10-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

#define		NOTE_UNAVAIL	-1

extern int cur_groupnum;

char note_h_path[LEN];			/* Path:	*/
char note_h_date[LEN];			/* Date:	*/
char note_h_subj[LEN];			/* Subject:	*/
char note_h_from[LEN];			/* From:	*/
char note_h_org[LEN];			/* Organization: */
char note_h_newsgroups[LEN];	/* Newsgroups:	*/
char note_h_messageid[LEN];		/* Message-ID:	*/
char note_h_distrib[LEN];		/* Distribution: */
char note_h_followup[LEN];		/* Followup-To: */

char note_full_name[100];
char note_from_addr[100];
char *glob_page_group;

FILE *note_fp;					/* the body of the current article */

int	note_line;
int	note_page;					/* what page we're on */
int	note_end;					/* we're done showing this article */
int	rotate;						/* 0=normal, 13=rot13 decode */
int last_resp;					/* current & previous article for - command */
int this_resp;
int glob_respnum;

long note_mark[MAX_PAGES];		/* ftells on beginnings of pages */
long note_size;					/* stat size in bytes of article */


int show_page (respnum, group, group_path)
	int respnum;
	char *group;
	char *group_path;
{
	char ch;
	int i, n;
	int kill_state = NO_KILLING;
	int old_sort_art_type = sort_art_type;
	int old_top;
	long old_artnum;
	long art;

restart:

	glob_respnum = respnum;
	glob_page_group = group;

#ifdef SIGTSTP
	if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
		sigemptyset (&page_act.sa_mask);
		page_act.sa_flags = SA_RESTART | SA_RESETHAND;
		page_act.sa_handler = page_suspend;
		sigaction (SIGTSTP, &page_act, 0L);
#else
		signal (SIGTSTP, page_suspend);
#endif
	}
#endif

#ifdef SIGWINCH
	signal (SIGWINCH, page_resize);
#endif

	if (respnum != this_resp) {	   /* remember current & previous */
		last_resp = this_resp;	   /* articles for - command */
		this_resp = respnum;
	}

	rotate = 0;			/* normal mode, not rot13 */
	art = arts[respnum].artnum;
	arts[respnum].unread = ART_READ;	/* mark article as read */
	art_open (art, group_path);

	if (note_page == NOTE_UNAVAIL) {
		ClearScreen ();
		printf (txt_art_unavailable, art);
		fflush (stdout);
	} else {
		show_note_page (respnum, group);
	}

	while (1) {
		ch = ReadCh();

		if (ch >= '0' && ch <= '9') {

			n = prompt_response (ch, respnum);
			if (n != -1) {
				respnum = n;
				goto restart;
			}

		} else switch (ch) {
			case ESC:
				ch = ReadCh ();
				if (ch == '[' || ch == 'O')
					ch = ReadCh ();
				switch (ch) {
				case 'G':		/* ansi  PgDn */
				case 'U':		/* at386 PgDn */
				case 'T':		/* 97801 PgDn */
					goto page_down;

				case 'I':		/* ansi  PgUp */
				case 'V':		/* at386 PgUp */
				case 'S':		/* 97801 PgUp */
				case 'v':		/* emacs style */
					goto page_up;

				case 'H':		/* at386 Home */
					goto begin_of_article;

				case 'F':		/* ansi End */
				case 'Y':		/* at386 End */
					goto end_of_article;
				}
				break;

			case '!':
				shell_escape ();
				redraw_page (respnum, group);
				break;

			case '$':	/* goto end of article */
			case 'G':	/* 'less' compatible */
end_of_article:			
				if (show_last_page ()) {
					show_note_page (respnum, group);
				}
				break;

			case '-':	/* show last viewed article */
				if (last_resp < 0) {
					info_message(txt_no_last_message);
					break;
				}
				art_close();
				respnum = last_resp;
				goto restart;

			case '|':	/* pipe article/thread/tagged arts to command */
				set_real_uid_gid ();
				feed_articles (FEED_PIPE, PAGE_LEVEL, "Pipe", respnum, group_path);
				set_tin_uid_gid ();
				break;

			case '/':	/* search forwards in article */
				if (search_article (TRUE)) {
					show_note_page (respnum, group);
				}
				break;

			case '<':	/* goto first article in current thread */
				n = which_base (respnum);
				if (n != respnum && n >= 0) {
					respnum = n;
					art_close ();
					goto restart;
				}
				break;

			case '>':	/* goto last article in current thread */
				for (i = respnum; i >= 0; i = arts[i].thread) {
					n = i;
				}
				if (n != respnum) {
					respnum = n;
					art_close ();
					goto restart;
				}
				break;

			case ' ': 			/* next page or response */
			case ctrl('D'):		/* vi style */
			case ctrl('V'):		/* emacs style */
page_down:
				if (note_page == NOTE_UNAVAIL) {
					n = next_response (respnum);
					if (n == -1) {
						return (which_base (respnum));
					}
					respnum = n;
					goto restart;
				} else if (note_end) {
					art_close ();
					n = next_response (respnum);
					if (n == -1) {
						return (which_base (respnum));
					}
					respnum = n;
					goto restart;
				} else
					show_note_page (respnum, group);
				break;

			case '\r':
			case '\n':	/* go to start of next thread */
				art_close ();
				n = next_basenote (respnum);
				if (n == -1)
					return (which_base (respnum));

				respnum = n;
				goto restart;

			case '\t': 	/* next page or unread response */
				if (note_page == NOTE_UNAVAIL) {
					n = next_unread (next_response (respnum));
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;

				} else if (note_end) {
					art_close();
					n = next_unread(next_response(respnum));
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;
				} else
					show_note_page(respnum, group);
				break;

			case ctrl('H'):	/* show article headers */
				if (note_page == NOTE_UNAVAIL) {
					n = next_response (respnum);
					if (n == -1)
						return (which_base (respnum));

					respnum = n;
					goto restart;
				} else {
					note_page = 0;
					note_end = FALSE;
					fseek(note_fp, 0L, 0);
					show_note_page(respnum, group);
				}
				break;

			case ctrl('K'):		/* kill article */
				if (kill_articles) {
					if (kill_art_menu (group, respnum)) {
						if (kill_any_articles (group)) {
							reload_index_file (group, TRUE);
						}
					}
					redraw_page(respnum, group);
				} else {
					info_message (txt_switch_on_kill_art_menu);
				}
				break;

			case ctrl('L'):		/* redraw current page of article */
				redraw_page (respnum, group);
				break;

			case ctrl('R'):		/* redraw beginning of article */
			case 'g':			/* 'less' compatible */
begin_of_article:			
				if (note_page == NOTE_UNAVAIL) {
					ClearScreen();
					printf(txt_art_unavailable,arts[respnum].artnum);
					fflush(stdout);
				} else {
					note_page = 0;
					note_end = FALSE;
					fseek(note_fp, note_mark[0], 0);
					show_note_page(respnum, group);
				}
				break;

			case ctrl('X'):
			case '%':
			case 'd':	/* toggle rot-13 mode */
				if (rotate)
					rotate = 0;
				else
					rotate = 13;
				redraw_page (respnum, group);
				info_message (txt_toggled_rot13);
				break;

			case 'a':	/* author search forward */
			case 'A':	/* author search backward */
				i = (ch == 'a');
				n = search_author (respnum, i);
				if (n < 0)
					break;
				respnum = n;
				goto restart;
				/* NOTREACHED */

			case ctrl('U'):
			case 'b':		/* back a page */
page_up:
				if (note_page == NOTE_UNAVAIL) {
					art_close();
					n = prev_response (respnum);
					if (n == -1)
						return (which_resp (respnum));

					respnum = n;
					goto restart;

				} else {
					if (note_page <= 1) {
						info_message (txt_begin_of_art);
					} else {
						note_page -= 2;
						note_end = FALSE;
						fseek (note_fp, note_mark[note_page], 0);
						show_note_page (respnum, group);
					}
				}
				break;

			case 'B':	/* bug/gripe/comment mailed to author */
				mail_bug_report ();
				redraw_page (respnum, group);
				break;
				
			case 'c':	/* catchup--mark all articles as read */
			    if (prompt_yn (LINES, txt_mark_all_read, 'y')) {
					for (n = 0; n < top; n++) {
						arts[n].unread = ART_READ;
					}
					fix_new_highest(cur_groupnum);
					if (cur_groupnum + 1 < local_top) {
						cur_groupnum++;
					}
					art_close();
					return -1;
			    }
				break;

			case 'C':	/* cancel an article */
				if (cancel_article ()) {
					redraw_page (respnum, group);
				}
				break;
	
			case 'f':	/* post a followup to this article */
#ifdef USE_NNTP
				if (! can_post) {
					info_message (txt_cannot_post);
					break;
				}
#endif
				if (post_response (group, FALSE)) {
					update_newsrc (group, my_group[cur_groupnum], FALSE);
					n = which_base (respnum);
					art_close ();
					index_group (group, group_path);
					read_newsrc_line (group);
					respnum = choose_resp (n, nresp(n));
					goto restart;
				} else
					redraw_page (respnum, group);
				break;

			case 'F':	/* post a followup to this article */
#ifdef USE_NNTP
				if (! can_post) {
					info_message (txt_cannot_post);
					break;
				}
#endif
				if (post_response (group, TRUE)) {
					update_newsrc (group, my_group[cur_groupnum], FALSE);
					n = which_base (respnum);
					art_close ();
					index_group (group, group_path);
					read_newsrc_line (group);
					respnum = choose_resp (n, nresp(n));
					goto restart;
				} else
					redraw_page (respnum, group);
				break;

			case 'h':	/* help overview */
				show_info_page (HELP_INFO, help_page, txt_art_pager_com);
				redraw_page (respnum, group);
				break;

			case 'H':	/* help in-depth */
				help_page_info ();
				redraw_page(respnum, group);
				break;

			case 'i':	/* return to index page */
return_to_index:
				art_close ();
				if (kill_state == NO_KILLING &&
					sort_art_type != old_sort_art_type) {
					make_threads (TRUE);
					find_base ();
				}
				if (kill_state == KILLING) {
					old_top = top;
					old_artnum = arts[respnum].artnum;
					if (kill_articles) {
						kill_any_articles (group);
						reload_index_file (group, TRUE);	/* kill arts */
					} else {
						reload_index_file (group, FALSE);	/* unkill arts */
					}
					return find_new_pos (old_top, old_artnum, respnum);
				}
				return (which_base (respnum));

			case 'I':	/* toggle inverse video */
				inverse_okay = !inverse_okay;
				if (inverse_okay) {
					info_message (txt_inverse_on);
				} else {
					draw_arrow_mark = TRUE;
					info_message (txt_inverse_off);
				}
				redraw_page (respnum, group);
				break;

			case 'k':
				if (note_page == NOTE_UNAVAIL) {
					n = next_unread (next_response(respnum));
					if (n == -1)
						return (which_base (respnum));
				} else {
					art_close ();
					n = next_unread (next_response (respnum));
					if (n == -1)
						return (which_base (respnum));
				}
				respnum = n;
				goto restart;
				/* NOTREACHED */

			case 'K':	/* mark rest of thread as read */
				for (n = respnum; n >= 0; n = arts[n].thread)
					arts[n].unread = ART_READ;
				n = next_unread (next_response (respnum));
				if (n == -1)
					goto return_to_index;
				art_close ();
				respnum = n;
				goto restart;
				/* NOTREACHED */

			case 'm':	/* mail article/thread/tagged articles to somebody */
				set_real_uid_gid ();
				feed_articles (FEED_MAIL, PAGE_LEVEL, "Mail", respnum, group_path);
				set_tin_uid_gid ();
				break;

			case 'M':	/* options menu */
				if (change_rcfile (group, FALSE) == KILLING) {
					kill_state = KILLING;
				} 
				redraw_page (respnum, group);
			    break;

			case 'n':	/* skip to next article */
				art_close ();
				n = next_response (respnum);
				if (n == -1)
					return (which_base(respnum));

				respnum = n;
				goto restart;
				/* NOTREACHED */
				
			case 'N':	/* next unread article */
				n = next_unread(next_response(respnum));
				if (n == -1)
					info_message(txt_no_next_unread_art);
				else {
					art_close();
					respnum = n;
					goto restart;
				}
				break;

			case 'o':	/* output art/thread/tagged arts to printer */
				set_real_uid_gid ();
				feed_articles (FEED_PRINT, PAGE_LEVEL, "Print", respnum, group_path);
				set_tin_uid_gid ();
				break;

			case 'p':	/* previous article */
				art_close ();
				n = prev_response (respnum);
				if (n == -1)
					return (which_resp (respnum));

				respnum = n;
				goto restart;

			case 'P':	/* previous unread article */
				n = prev_unread (prev_response (respnum));
				if (n == -1)
				    info_message (txt_no_prev_unread_art);
				else {
					art_close ();
					respnum = n;
					goto restart;
				}
				break;

			case 'q':	/* quit */
				if (prompt_yn (LINES, txt_quit, 'y')) {
					return -2;
				}
				break;
	
			case 'r':	/* reply to author through mail */
				mail_to_author (FALSE);
				redraw_page (respnum, group);
				break;

			case 'R':	/* reply to author, copy text */
				mail_to_author (TRUE);
				redraw_page (respnum, group);
				break;

			case 's':	/* save article/thread/tagged articles */
				set_real_uid_gid ();
				feed_articles (FEED_SAVE, PAGE_LEVEL, "Save", respnum, group_path);
				set_tin_uid_gid ();
				break;

			case 't':	/* return to group selection page */
				art_close ();
				if (kill_state == KILLING) {
					if (kill_articles) {
						kill_any_articles (group);
						reload_index_file (group, TRUE);	/* kill arts */
					} else {
						reload_index_file (group, FALSE);	/* unkill arts */
					}
				}
				update_newsrc (group, my_group[cur_groupnum], FALSE);
				fix_new_highest (cur_groupnum);
				return -1;

			case 'T':	/* tag/untag article for saving */
				if (arts[respnum].tagged) {
					arts[respnum].tagged = 0;
					info_message (txt_untagged_art);
				} else {
					arts[respnum].tagged = ++num_of_tagged_files;
					info_message (txt_tagged_art);
				}
				break;

			case 'v':
				info_message (cvers);
				break;

			case 'w':	/* post a basenote */
#ifdef USE_NNTP
				if (! can_post) {
					info_message (txt_cannot_post);
					break;
				}
#endif
				if (post_base (group)) {
					update_newsrc (group, my_group[cur_groupnum], FALSE);
					index_group (group, group_path);
					read_newsrc_line (group);
					redraw_page (respnum, group);
				}
				break;

			case 'W':	/* display messages posted by user */
				if (user_posted_messages ()) {
					redraw_page(respnum, group);
				}
				break;

			case 'z':	/* mark article as unread (to return) */
				arts[respnum].unread = ART_WILL_RETURN;
				info_message(txt_art_marked_as_unread);
				break;

			default:
			    info_message(txt_bad_command);
		}
	}
}


void redraw_page (respnum, group)
	int respnum;
	char *group;
{

	if (note_page == NOTE_UNAVAIL) {
		ClearScreen ();
		printf (txt_art_unavailable, arts[respnum].artnum);
		fflush (stdout);
	} else if (note_page > 0) {
		note_page--;
		fseek (note_fp, note_mark[note_page], 0);
		show_note_page (respnum, group);
	}
}


void show_note_page (respnum, group)
	int respnum;
	char *group;
{
	char buf[LEN];
	char buf2[LEN+50];
	char *p, *q;
	int i, j;
	int ctrl_L;		/* form feed character detected */
	long tmp_pos;
	ClearScreen ();

	note_line = 1;

	if (note_size == 0L) {
		tmp_pos = ftell (note_fp);
		fseek (note_fp, 0L, 2);			/* goto end of article */
		note_size = ftell (note_fp);
		fseek (note_fp, tmp_pos, 0);	/* goto old position */
	}
	
	if (note_page == 0)
		show_first_header (respnum, group);
	else
		show_cont_header (respnum);

	ctrl_L = FALSE;
	while (note_line < LINES) {
		if (fgets(buf, sizeof buf, note_fp) == NULL) {
			note_end = TRUE;
			break;
		}

		buf[LEN-1] = '\0';
		if (rotate)
			for (p = buf, q = buf2;
					*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == 12) {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j)
						*q++ = ' ';
				} else if (((*p) & 0xFF) < ' ') {
					*q++ = '^';
					*q++ = ((*p) & 0xFF) + '@';
				} else if (*p >= 'A' && *p <= 'Z')
					*q++ = 'A' + (*p - 'A' + rotate) % 26;
				else if (*p >= 'a' && *p <= 'z')
					*q++ = 'a' + (*p - 'a' + rotate) % 26;
				else
					*q++ = *p;
			}
		else
			for (p = buf, q = buf2;
					*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == 12) {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j)
						*q++ = ' ';
				} else if (((*p) & 0xFF) < ' ') {
					*q++ = '^';
					*q++ = ((*p) & 0xFF) + '@';
				} else
					*q++ = *p;
			}

		*q = '\0';

		printf("%s\r\n", buf2);

		note_line += ((int) strlen (buf2) / COLS) + 1;

		if (ctrl_L) {
			break;
		}
	}

	note_mark[++note_page] = ftell (note_fp);

	if (note_mark[note_page] == note_size) {
		note_end = TRUE;
	}

	if (note_end) {
		MoveCursor (LINES, MORE_POS-(5+BLANK_PAGE_COLS));
		StartInverse ();	
		if (arts[respnum].thread != -1) {
			printf (txt_next_resp);
			fflush (stdout);
		} else {
			printf (txt_last_resp);
			fflush (stdout);
		}
		EndInverse ();
	} else {
		if (note_size > 0) {
			draw_percent_mark ((int) note_mark[note_page], (int) note_size);
		} else {
			MoveCursor (LINES, MORE_POS-BLANK_PAGE_COLS);
			StartInverse ();	
		    printf (txt_more);
			fflush (stdout);
			EndInverse ();
		}
	}
	MoveCursor (LINES, 0);
}


void show_first_header (respnum, group)
	int respnum;
	char *group;
{
	int whichresp;
	int x_resp;
	char buf[LEN];
	char tmp[LEN];
	int pos, i;
	int n;

	whichresp = which_resp (respnum);
	x_resp = nresp (which_base (respnum));

	ClearScreen ();

	strcpy (buf, note_h_date);
	pos = (COLS - (int) strlen (group)) / 2;
	for (i = strlen(buf); i < pos; i++)
		buf[i] = ' ';
	buf[i] = '\0';

	strcat (buf, group);

	for (i = strlen(buf); i < RIGHT_POS ; i++)
		buf[i] = ' ';
	buf[i] = '\0';

	printf (txt_thread_x_of_n, buf, which_base (respnum) + 1, top_base);

	sprintf (buf, txt_art, arts[respnum].artnum);
	n = strlen (buf);
	fputs (buf, stdout);

	strcpy (buf, note_h_subj);
	buf[RIGHT_POS - 5 - n] = '\0';

	pos = ((COLS - (int) strlen (buf)) / 2) - 2;

	if (pos > n) {
		MoveCursor (1, pos);
	} else {
		MoveCursor (1, n);
	}

	StartInverse ();
	fputs (buf, stdout);
	EndInverse ();

	MoveCursor (1, RIGHT_POS);
	if (whichresp)
		printf (txt_resp_x_of_n, whichresp, x_resp);
	else {
		if (x_resp == 0)
			printf (txt_no_resp);
		else if (x_resp == 1)
			printf (txt_1_resp);
		else
			printf (txt_x_resp, x_resp);
	}

	if (*note_h_org)
		sprintf (tmp, txt_s_at_s, note_full_name, note_h_org);
	else
		strcpy (tmp, note_full_name);

	tmp[LEN] = '\0';

	sprintf (buf, "%s  ", note_from_addr);

	pos = COLS - 1 - (int) strlen(tmp);
	if ((int) strlen (buf) + (int) strlen (tmp) >= COLS - 1) {
		strncat (buf, tmp, COLS - 1 - (int) strlen(buf));
		buf[COLS - 1] = '\0';
	} else {
		for (i = strlen(buf); i < pos; i++)
			buf[i] = ' ';
		buf[i] = '\0';
		strcat (buf, tmp);
	}
	printf ("%s\r\n\r\n", buf);

	note_line += 4;
}


void show_cont_header (respnum)
	int respnum;
{
	int whichresp;
	int whichbase;
	char buf[LEN];

	whichresp = which_resp (respnum);
	whichbase = which_base (respnum);

	assert (whichbase < top_base);

	if (whichresp)
		sprintf(buf, txt_thread_resp_page,
			whichbase + 1,
			top_base,
			whichresp,
			note_page + 1,
			note_h_subj);
	else
		sprintf(buf, txt_thread_page,
			whichbase + 1,
			top_base,
			note_page + 1,
			note_h_subj);

	buf[COLS-1] = '\0';
	printf("%s\r\n\r\n", buf);

	note_line += 2;
}


void art_open (art, group_path)
	long art;
	char *group_path;
{
	char buf[1025];
	char *p;

	note_page = 0;

	if ((note_fp = open_art_fp (group_path, art)) == NULL) {
		note_page = NOTE_UNAVAIL;
		return;
	}

	note_h_from[0] = '\0';
	note_h_path[0] = '\0';
	note_h_subj[0] = '\0';
	note_h_org[0] = '\0';
	note_h_date[0] = '\0';
	note_h_newsgroups[0] = '\0';
	note_h_messageid[0] = '\0';
	note_h_distrib[0] = '\0';
	note_h_followup[0] = '\0';

	while (fgets(buf, sizeof buf, note_fp) != NULL) {
		buf[1024] = '\0';

		for (p=buf ; *p && *p != '\n' ; p++) {
			if (((*p) & 0xFF) < ' ')
				*p = ' ';
		}
		*p = '\0';
		
		if (*buf == '\0')
			break;

  		if (match_header (buf, "From", note_h_from, LEN))
  			continue;
  		if (match_header (buf, "Path", note_h_path, LEN))
  			continue;
  		if (match_header (buf, "Subject", note_h_subj, LEN))
  			continue;
  		if (match_header (buf, "Organization", note_h_org, LEN))
  			continue;
  		if (match_header (buf, "Date", note_h_date, LEN))
  			continue;
  		if (match_header (buf, "Newsgroups", note_h_newsgroups, LEN))
  			continue;
  		if (match_header (buf, "Message-ID", note_h_messageid, LEN))
  			continue;
  		if (match_header (buf, "Message-Id", note_h_messageid, LEN))
  			continue;
  		if (match_header (buf, "Distribution", note_h_distrib, LEN))
  			continue;
  		if (match_header (buf, "Followup-To", note_h_followup, LEN))
  			continue;
	}

	note_page = 0;
	note_mark[0] = ftell (note_fp);

	parse_from (note_h_from, note_from_addr, note_full_name);
	note_end = FALSE;

	return;
}


void art_close ()
{
	if (note_page != NOTE_UNAVAIL)
		fclose (note_fp);
}


int prompt_response(ch, respnum)
	int respnum;
{
	int num;

	clear_message ();

	if ((num = parse_num (ch, txt_read_resp)) == -1) {
		clear_message ();
		return -1;
	}

	return choose_resp (which_base (respnum), num);
}

/*
 *  return response number n from thread i
 */

int choose_resp (i, n)
	int i;
	int n;
{
	int j;

	j = (int) base[i];

	while (n-- && arts[j].thread >= 0) {
		j = arts[j].thread;
	}

	return j;
}


/*
 *  Parse various From: lines into the component mail addresses and
 *  real names
 */

void parse_from (str, addr, name)
	char *str;
	char *addr;
	char *name;
{
	char *p;
	
	for (p=str ; *p ; p++) {
		if (((*p) & 0xFF) < ' ')
			*p = ' ';
	}

	while (*str && *str != ' ')
		*addr++ = *str++;
	*addr = '\0';
	if (*str++ == ' ') {
		if (*str++ == '(') {
			if (*str == '"')
				str++;  /* Kill "quotes around names"         */
					/* But don't touch quotes inside the  */
					/* Name (that's what that nonsense    */
					/* below is for			      */
			while (*str && *str != ')' && !(*str=='"'&&str[1]==')'))
				*name++ = *str++;
		}
	}
	*name = '\0';
}


/*
 *  Find the previous response.  Go to the last response in the previous
 *  thread if we go past the beginning of this thread.
 */

int prev_response (n)
	int n;
{
	int resp;
	int i;

	resp = which_resp (n);

	if (resp > 0)
		return choose_resp (which_base (n), resp-1);

	i = which_base (n) - 1;

	if (i < 0)
		return -1;

	return choose_resp (i, nresp (i));
}


/*
 *  Find the next response.  Go to the next basenote if there
 *  are no more responses in this thread
 */

int next_response (n)
	int n;
{
	int i;

	if (arts[n].thread >= 0)
		return arts[n].thread;

	i = which_base (n) + 1;

	if (i >= top_base)
		return -1;

	return (int) base[i];
}


/*
 *  Given a respnum (index into arts[]), find the respnum of the
 *  next basenote
 */

int next_basenote (n)
	int n;
{
	int i;

	i = which_base (n) + 1;
	if (i >= top_base)
		return -1;

	return (int) base[i];
}


void yank_to_addr (orig, addr)
	char *orig;
	char *addr;
{
	char *p;

	for (p = orig; *p; p++)
		if (((*p) & 0xFF) < ' ')
			*p = ' ';

	while (*addr)
		addr++;

	while (*orig) {
		while (*orig && (*orig == ' ' || *orig == '"' || *orig == ','))
			orig++;
		*addr++ = ' ';
		while (*orig && (*orig != ' ' && *orig != ',' && *orig != '"'))
			*addr++ = *orig++;
		while (*orig && (*orig == ' ' || *orig == '"' || *orig == ','))
			orig++;
		if (*orig == '(') {
			while (*orig && *orig != ')')
				orig++;
			if (*orig == ')')
				orig++;
		}
	}
	*addr = '\0';
}


int show_last_page ()
{
	char buf[LEN];
	char buf2[LEN+50];
	char *p, *q;
	int ctrl_L;		/* form feed character detected */
	int i, j;
	long tmp_pos;
	
	if (note_end) {
		return FALSE;
	}

	if (note_size == 0L) {
		tmp_pos = ftell (note_fp);
		fseek (note_fp, 0L, 2);			/* goto end of article */
		note_size = ftell (note_fp);
		fseek (note_fp, tmp_pos, 0);	/* goto old position */
	}

	while (! note_end) {
		note_line = 1;
		ctrl_L = FALSE;

		if (note_page == 0) {
			note_line += 4;
		} else {
			note_line += 2;
		}
		while (note_line < LINES) {
			if (fgets (buf, sizeof buf, note_fp) == NULL) {
				note_end = TRUE;
				break;
			}
			buf[LEN-1] = '\0';
			for (p = buf, q = buf2;	*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == 12) {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j) {
						*q++ = ' ';
					}
				} else if (((*p) & 0xFF) < ' ') {
					*q++ = '^';
					*q++ = ((*p) & 0xFF) + '@';
				} else {
					*q++ = *p;
				}
			}
			*q = '\0';
			note_line += ((int) strlen (buf2) / COLS) + 1;

			if (ctrl_L) {
				break;
			}
		}
		if (note_mark[note_page] == note_size) {
			note_end = TRUE;
			note_page--;
			break;
		} else if (! note_end) {
			note_mark[++note_page] = ftell(note_fp);
		}
	}
	fseek (note_fp, note_mark[note_page], 0);
	return TRUE;
}


int match_header (buf, pat, body, len)
	char *buf;
	char *pat;
	char *body;
	int	len;
{
	int	plen = strlen (pat);

	if(strncmp (buf, pat, plen) == 0 && buf[plen] == ':' && buf[plen + 1] == ' ') {
		plen += 2;
		while (buf[plen] == ' ')
			plen++;
		strncpy (body, &buf[plen], len);
		body[len - 1] = '\0';
		return TRUE;
	}
	return FALSE;
}
