/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : post.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 16-10-91
 *  Release   : 1.0
 *  Notes     : mailing/posting/replying/followup & cancel article routines
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char note_h_distrib[LEN];		/* Distribution: */
extern char note_h_followup[LEN];		/* Followup-To: */
extern char note_h_from[LEN];			/* From:	*/
extern char note_h_messageid[LEN];		/* Message-ID:	*/
extern char note_h_newsgroups[LEN];		/* Newsgroups:	*/
extern char note_h_subj[LEN];			/* Subject:	*/

extern char note_from_addr[100];
extern char note_full_name[100];

extern FILE *note_fp;					/* the body of the current article */

extern long note_mark[MAX_PAGES];		/* ftells on beginnings of pages */

char default_post_subject[LEN];		/* offers user default choice */

struct posted_t *posted;


int user_posted_messages ()
{
	char buf[LEN];
	FILE *fp;
	int i, j, k;
	int no_of_lines = 0;

	set_real_uid_gid ();
		
	if ((fp = fopen (postfile, "r")) == NULL) {
		clear_message ();
		set_tin_uid_gid ();
		return FALSE;
	} else {
		while (fgets (buf, sizeof (buf), fp) != NULL) {
			no_of_lines++;
		}
		if (! no_of_lines) {
			fclose (fp);
			info_message (txt_no_arts_posted);
			return FALSE;
		}
		rewind (fp);
		posted = (struct posted_t *) my_malloc ((unsigned) (no_of_lines+1) * sizeof (struct posted_t));
		for (i=0 ; fgets (buf, sizeof (buf), fp) != NULL ; i++) {
			for (j=0 ; buf[j] != '|' && buf[j] != '\n' ; j++) {
				posted[i].date[j] = buf[j];		/* posted date */
			}
			if (buf[j] == '\n') {	
				error_message ("Corrupted file %s", postfile);
				sleep (1);
				fclose (fp);
				clear_message ();
				return FALSE;
			}
			posted[i].date[j++] = '\0';
			for (k=j,j=0 ; buf[k] != '|' && buf[k] != ',' ; k++, j++) {
				posted[i].group[j] = buf[k];
			}
			if (buf[k] == ',') {
				while (buf[k] != '|' && buf[k] != '\n') {
					k++;
				}
				posted[i].group[j++] = ',';
				posted[i].group[j++] = '.';
				posted[i].group[j++] = '.';
				posted[i].group[j++] = '.';
			}
			posted[i].group[j++] = '\0';
			k++;
			for (j=k,k=0 ; buf[j] != '\n' ; j++, k++) {
				posted[i].subj[k] = buf[j];
			}
			posted[i].subj[k++] = '\0';
		}
		fclose (fp);
		set_tin_uid_gid ();

		show_info_page (POST_INFO, (char **) 0, txt_post_history_menu);
		if (posted != (struct posted_t *) 0) {
			free (posted);
			posted = (struct posted_t *) 0;
		}
		return TRUE;
	}
}


void update_art_posted_file (group, subj)
	char *group;
	char *subj;
{
	char buf[LEN];
	char tmp_post[LEN];
	FILE *fp, *tmp_fp;
	long epoch;
	struct tm *tm;

	sprintf (tmp_post, "%s.%d", postfile, getpid ());

	set_real_uid_gid ();

	if ((tmp_fp = fopen (tmp_post, "w")) != NULL) {
		time (&epoch);
		tm = localtime (&epoch);
		fprintf (tmp_fp, "%02d-%02d-%02d|%s|%s\n",
			tm->tm_mday, tm->tm_mon+1, tm->tm_year, group, subj);
		fclose (tmp_fp);
	}

	if ((tmp_fp = fopen (tmp_post, "a+")) != NULL) {
		if ((fp = fopen (postfile, "r")) != NULL) {
			while (fgets (buf, sizeof buf, fp) != NULL) {
				fprintf (tmp_fp, "%s", buf);
			}	
			fclose (fp);
			rename_file (tmp_post, postfile);
		}
		fclose (tmp_fp);
	}
	set_tin_uid_gid ();
}

/*
 *  Post an original article (not a followup)
 */

int post_base (group)
	char *group;
{
	FILE *fp;
	char ch;
	char ch_default = 'p';
	char subj[LEN];
	char buf[LEN];
	int redraw_screen = FALSE;

	start_line_offset = 4;

	sprintf (msg, txt_post_subject, default_post_subject);
	
	if (! parse_string (msg, subj)) {
		clear_message ();
		return (redraw_screen);
	}

	if (strlen (subj)) {
		my_strncpy (default_post_subject, subj, LEN);
	} else {
		if (default_post_subject[0]) {
			my_strncpy (subj, default_post_subject, LEN);
		} else {
			info_message (txt_no_subject);
			return (redraw_screen);
		}
	}
	
	wait_message (txt_post_an_article);

	set_real_uid_gid ();

	if ((fp = fopen (article, "w")) == NULL) {
		error_message (txt_cannot_open, article);
		set_tin_uid_gid ();
		return (redraw_screen);
	}
	chmod (article, 0600);

	fprintf (fp, "Subject: %s\n", subj);
	fprintf (fp, "Newsgroups: %s\n", group);
	fprintf (fp, "Distribution: \n");
	if (*my_org) {
		fprintf (fp, "Organization: %s\n", my_org);
		start_line_offset++;
	}
	if (*reply_to) {
		fprintf (fp, "Reply-To: %s\n", reply_to);
		start_line_offset++;
	}
	fprintf (fp, "\n");

	add_signature (fp, FALSE);
	fclose (fp);

	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			invoke_editor (article);
			redraw_screen = TRUE;
			break;

		case 'a':
		case ESC:
			unlink (article);
			clear_message ();
			set_tin_uid_gid ();
			return (redraw_screen);

		case 'p':
			wait_message (txt_posting);
			if (submit_file (article)) {
				unlink (article);
				info_message (txt_art_posted);
				goto post_base_done;
			} else {
				rename_file (article, dead_article);
				sprintf (buf, txt_art_rejected, dead_article);
				info_message (buf);
				sleep (3);
				set_tin_uid_gid ();
				return (redraw_screen);
			}
		}

		do {
			sprintf (msg, "%s%c", txt_abort_edit_post, ch_default);
			wait_message (msg);
			MoveCursor (LINES, (int) strlen (txt_abort_edit_post));
			if ((ch = ReadCh ()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 'p');
	}

post_base_done:
	set_tin_uid_gid ();
	update_art_posted_file (group, subj);
	return (redraw_screen);
}


int post_response (group, respnum)
	char *group;
	int respnum;
{
	FILE *fp;
	char ch, *ptr;
	char ch_default = 'p';
	char buf[LEN];
	int redraw_screen = FALSE;

	start_line_offset = 4;

	wait_message (txt_post_a_followup);
	
	if (*note_h_followup && strcmp (note_h_followup, "poster") == 0) {
		clear_message ();
		if (! prompt_yn (LINES, txt_resp_to_poster, 'y')) {
			return (redraw_screen);
		}
		*note_h_followup = '\0';
	} else if (*note_h_followup && strcmp(note_h_followup, group) != 0) {
	    MoveCursor (LINES/2, 0);
	    CleartoEOS ();
		center_line ((LINES/2)+2, TRUE, txt_resp_redirect);
	    MoveCursor ((LINES/2)+4, 0);

		printf ("    ");
		ptr = note_h_followup;
		while (*ptr) {
			if (*ptr != ',') {
				putc (*ptr, stdout);
			} else {
				printf ("\r\n    ");
			}
			fflush (stdout);
			ptr++;
		}

		if (! prompt_yn (LINES, txt_continue, 'y')) {
			return (redraw_screen);
		}
	}

	set_real_uid_gid ();

	if ((fp = fopen (article, "w")) == NULL) {
		error_message (txt_cannot_open, article);
		set_tin_uid_gid ();
		return (redraw_screen);
	}
	chmod (article, 0600);

	fprintf (fp, "Subject: Re: %s\n", eat_re (note_h_subj));

	if (*note_h_followup && strcmp(note_h_followup, "poster") != 0) {
		fprintf (fp, "Newsgroups: %s\n", note_h_followup);
	} else {
		fprintf (fp, "Newsgroups: %s\n", note_h_newsgroups);
	}
	if (note_h_distrib != '\0') {
		fprintf (fp, "Distribution: %s\n", note_h_distrib);
		start_line_offset++;
	}
	fprintf (fp, "References: %s\n", note_h_messageid);

	if (*my_org) {
		fprintf (fp, "Organization: %s\n", my_org);
		start_line_offset++;
	}
	if (*reply_to) {
		fprintf (fp, "Reply-To: %s\n", reply_to);
		start_line_offset++;
	}
	fprintf (fp, "\n");

	if (respnum) {		/* if "copy_text" */
		if (note_h_from[0]) { 	
			fprintf (fp, txt_writes, note_h_from);
		}
		fseek (note_fp, note_mark[0], 0);
		copy_fp (note_fp, fp, DEFAULT_COMMENT);
	}

	add_signature (fp, FALSE);
	fclose (fp);

	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			invoke_editor (article);
			redraw_screen = TRUE;
			break;

		case 'a':
		case ESC:
			unlink (article);
			clear_message ();
			set_tin_uid_gid ();
			return (redraw_screen);

		case 'p':
			wait_message (txt_posting);
			if (submit_file (article)) {
				unlink (article);
				info_message (txt_art_posted);
				goto post_response_done;
			} else {
				rename_file (article, dead_article);
				sprintf (buf, txt_art_rejected, dead_article);
				info_message (buf);
				sleep (3);
				set_tin_uid_gid ();
				return (redraw_screen);
			}
		}

		do {
			sprintf (msg, "%s%c", txt_abort_edit_post, ch_default);
			wait_message (msg);
			MoveCursor(LINES, (int) strlen (txt_abort_edit_post));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 'p');
	}

post_response_done:
	set_tin_uid_gid ();

	if (*note_h_followup && strcmp(note_h_followup, "poster") != 0)
		update_art_posted_file (note_h_followup, note_h_subj);
	else
		update_art_posted_file (note_h_newsgroups, note_h_subj);

	return (redraw_screen);
}


int mail_to_someone (address)
	char *address;
{
	char nam[100];
	char ch, ch_default = 's';
	char buf[LEN];
	char mail_to[LEN];
	FILE *fp;
	int redraw_screen = FALSE;

	start_line_offset = 4;
	
	strcpy (mail_to, address);
	clear_message ();
	
	set_real_uid_gid ();

	sprintf (nam, "%s/.letter", homedir);
	if ((fp = fopen (nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		set_tin_uid_gid ();
		return (redraw_screen);
	}
	chmod (nam, 0600);

	fprintf (fp, "To: %s\n", mail_to);
	fprintf (fp, "Subject: (fwd) %s\n", note_h_subj);
	if (*note_h_followup) {
		fprintf (fp, "Newsgroups: %s\n\n", note_h_followup);
	} else {
		fprintf (fp, "Newsgroups: %s\n", note_h_newsgroups);
	}
	if (*my_org) {
		fprintf (fp, "Organization: %s\n", my_org);
		start_line_offset++;
	}
	if (*reply_to) {
		fprintf (fp, "Reply-To: %s\n", reply_to);
		start_line_offset++;
	}
	fputs ("\n", fp);
	
	fseek (note_fp, 0L, 0);
	copy_fp (note_fp, fp, "");

	add_signature (fp, TRUE);
	fclose (fp);
	
	while (1) {
		do {
			my_strncpy (buf, note_h_subj, COLS-30);
			sprintf (msg, "%s [%s]: %c", txt_abort_edit_send, buf, ch_default);
			wait_message (msg);
			MoveCursor (LINES, (int) strlen (msg)-1);
			if ((ch = ReadCh ()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 's');

		switch (ch) {
		case 'e':
			invoke_editor (nam);
			redraw_screen = TRUE;
			break;

		case 'a':
		case ESC:
			unlink (nam);
			clear_message ();
			set_tin_uid_gid ();
			return (redraw_screen);

		case 's':
/*
 *  Open letter and get the To: line in case they changed it with
 *  the editor
 */
			find_new_to (nam, mail_to);
			sprintf (msg, txt_mailing_to, mail_to);
			wait_message (msg);
			sprintf (buf, "%s \"%s\" < %s", mailer, mail_to, nam);
			if (invoke_cmd (buf)) {
				info_message (txt_message_sent);
				goto mail_to_someone_done;
			} else {
				error_message (txt_command_failed_s, buf);
				break;
			}
		}
	}

mail_to_someone_done:
	unlink (nam);
	set_tin_uid_gid ();

	return (redraw_screen);
}


int mail_bug_report ()
{
	char nam[100];
	FILE *fp;
	char ch;
	char ch_default = 's';
	char buf[LEN];
	char mail_to[LEN];

	start_line_offset = 5;
	
	wait_message (txt_mail_bug_report);
	
	set_real_uid_gid ();

	sprintf (nam, "%s/.bugreport", homedir);
	if ((fp = fopen (nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		set_tin_uid_gid ();
		return FALSE;
	}
	chmod(nam, 0600);

	fprintf (fp, "To: %s\n", bug_addr);
	fprintf (fp, "Subject: BUG REPORT %s %s PL%d %s\n",	progname,
		version, PATCHLEVEL, (compiled_with_nntp ? "(NNTP)" : ""));
	if (*my_org) {
		fprintf (fp, "Organization: %s\n", my_org);
		start_line_offset++;
	}
	if (*reply_to) {
		fprintf (fp, "Reply-To: %s\n", reply_to);
		start_line_offset++;
	}
	fprintf (fp, "\nPlease enter the following information:\n");
	fprintf (fp, "1) machine:\n");
	fprintf (fp, "2) os type:\n");
	fprintf (fp, "\nPlease enter bug/gripe/comment report:\n");

	add_signature (fp, TRUE);
	fclose (fp);
	
	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			invoke_editor (nam);
			break;

		case 'a':
		case ESC:
			unlink (nam);
			clear_message ();
			set_tin_uid_gid ();
			return TRUE;

		case 's':
			strcpy (mail_to, bug_addr);
			find_new_to (nam, mail_to);
			sprintf (msg, txt_mailing_to, mail_to);
			wait_message (msg);
			sprintf (buf, "%s \"%s\" < %s", mailer, mail_to, nam);
			if (invoke_cmd (buf)) {
				info_message (txt_message_sent);
				goto mail_bug_report_done;
			} else {
				error_message (txt_command_failed_s, buf);
				break;
			}
		}

		do {
			sprintf (msg, "%s: %c", txt_abort_edit_send, ch_default);
			wait_message (msg);
			MoveCursor (LINES, (int) strlen (msg)-1);
			if ((ch = ReadCh ()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 's');
	}

mail_bug_report_done:
	unlink (nam);
	set_tin_uid_gid ();

	return TRUE;
}


int mail_to_author (copy_text)
	int copy_text;
{
	char buf[LEN];
	char nam[100];
	char mail_to[LEN];
	char ch, ch_default = 's';
	FILE *fp;
	int redraw_screen = FALSE;

	start_line_offset = 4;
	
	wait_message (txt_reply_to_author);

	set_real_uid_gid ();

	sprintf (nam, "%s/.letter", homedir);
	if ((fp = fopen (nam, "w")) == NULL) {
		error_message (txt_cannot_open, nam);
		set_tin_uid_gid ();
		return (redraw_screen);
	}
	chmod (nam, 0600);

	fprintf (fp, "To: %s%s (%s)\n", note_from_addr, add_addr, note_full_name);
	fprintf (fp, "Subject: Re: %s\n", eat_re(note_h_subj) );
	fprintf (fp, "Newsgroups: %s\n", note_h_newsgroups);
	if (*my_org) {
		fprintf (fp, "Organization: %s\n", my_org);
		start_line_offset++;
	}
	if (*reply_to) {
		fprintf (fp, "Reply-To: %s\n", reply_to);
		start_line_offset++;
	}
	fputs ("\n", fp);

	if (copy_text) {		/* if "copy_text" */
		fprintf (fp, txt_in_art_you_write, note_h_messageid);
		fseek (note_fp, note_mark[0], 0);
		copy_fp (note_fp, fp, DEFAULT_COMMENT);
	}

	add_signature (fp, TRUE);
	fclose (fp);

	ch = 'e';
	while (1) {
		switch (ch) {
		case 'e':
			invoke_editor (nam);
			redraw_screen = TRUE;
			break;

		case 'a':
		case ESC:
			unlink (nam);
			clear_message ();
			set_tin_uid_gid ();
			return (redraw_screen);

		case 's':
			strcpy (mail_to, note_from_addr);
			find_new_to (nam, mail_to);
			sprintf (msg, txt_mailing_to, mail_to);
			wait_message (msg);
			sprintf (buf, "%s \"%s\" < %s", mailer, mail_to, nam);
			if (invoke_cmd (buf)) {
				info_message (txt_message_sent);
				goto mail_to_author_done;
			} else {
				error_message (txt_command_failed_s, buf);
				break;
			}
		}

		do {
			sprintf (msg, "%s: %c", txt_abort_edit_send, ch_default);
			wait_message (msg);
			MoveCursor (LINES, (int) strlen (msg)-1);
			if ((ch = ReadCh ()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 's');
	}

mail_to_author_done:
	unlink (nam);
	set_tin_uid_gid ();

	return (redraw_screen);
}

/*
 *  Read a file grabbing the address given for To: and
 *  sticking it in mail_to
 */

void find_new_to (nam, mail_to)
	char *nam;
	char *mail_to;
{
	FILE *fp;
	char buf[LEN];
	char buf2[LEN];
	char new_mail_to[LEN];
	char *p;

	*new_mail_to = '\0';

	if ((fp = fopen(nam, "r")) == NULL) {
		fprintf(stderr, txt_cannot_open, nam);
		return;
	}

	while (fgets(buf, sizeof (buf), fp) != NULL) {
		for (p = buf; *p && *p != '\n'; p++)
			continue;
		*p = '\0';

		if (*buf == '\0')
			break;

		if (strncmp(buf, "To: ", 4) == 0) {
			strncpy(buf2, &buf[4], LEN);
			buf2[LEN-1] = '\0';
			yank_to_addr(buf2, new_mail_to);
		} else if (strncmp(buf, "Cc: ", 4) == 0) {
			strncpy(buf2, &buf[4], LEN);
			buf2[LEN-1] = '\0';
			yank_to_addr(buf2, new_mail_to);
		}
	}

	fclose (fp);
	if (new_mail_to[0] == ' ')
		my_strncpy (mail_to, &new_mail_to[1], LEN);
	else
		my_strncpy (mail_to, new_mail_to, LEN);
}


int cancel_article ()
{
	char ch, ch_default = 'c';
	char buf[LEN];
	char cancel[LEN];
	FILE *fp;
	int redraw_screen = FALSE;

	start_line_offset = 4;
	
	clear_message ();
	
	set_real_uid_gid ();

	sprintf (cancel, "%s/.cancel", homedir);
	if ((fp = fopen (cancel, "w")) == NULL) {
		error_message (txt_cannot_open, cancel);
		set_tin_uid_gid ();
		return (redraw_screen);
	}
	chmod (cancel, 0600);

	fprintf (fp, "Subject: cancel %s\n", note_h_messageid);
	if (*note_h_followup) {
		fprintf (fp, "Newsgroups: %s\n", note_h_followup);
	} else {
		fprintf (fp, "Newsgroups: %s\n", note_h_newsgroups);
	}
	fprintf (fp, "Control: cancel %s\n", note_h_messageid);
	if (*my_org) { 
		fprintf (fp, "Organization: %s\n", my_org);
		start_line_offset++;	
	}
	if (*reply_to) {
		fprintf (fp, "Reply-To: %s\n", reply_to);
		start_line_offset++;	
	}
	fputs ("\n", fp);

	fprintf (fp, "Article cancelled from within tin\n");
	
	fclose (fp);
	
	while (1) {
		do {
			my_strncpy (buf, note_h_subj, COLS-30);
			sprintf (msg, "%s [%s]: %c", txt_abort_edit_cancel, buf, ch_default);
			wait_message (msg);
			MoveCursor (LINES, (int) strlen (msg)-1);
			if ((ch = ReadCh ()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 'c');

		switch (ch) {
		case 'e':
			invoke_editor (cancel);
			redraw_screen = TRUE;
			break;

		case 'a':
		case ESC:
			unlink (cancel);
			clear_message ();
			set_tin_uid_gid ();
			return (redraw_screen);

		case 'c':
			wait_message (txt_cancelling);
			if (submit_file (cancel)) {
				info_message (txt_art_cancelled);
				goto cancel_article_done;
			} else {
				error_message (txt_command_failed_s, buf);
				break;
			}
		}
	}

cancel_article_done:
	unlink (cancel);
	set_tin_uid_gid ();

	return (redraw_screen);
}

int
submit_file(name)
char*	name;
{
	char	buf[LEN];
	char*	cp = buf;

#ifdef INEWSDIR
	strcpy(buf, INEWSDIR);
	strcat(buf, "/");
	cp = &buf[strlen(buf)];
#endif

	if (debug)
		sprintf (cp, "inews -h < %s", name);
	else
		sprintf (cp, "inews -h < %s > /dev/null 2>&1", name);

	if (invoke_cmd(buf))
		return TRUE;
	return FALSE;
}
