/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : prompt.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 30-09-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

/*
 *  parse_num
 *  get a number from the user
 *  Return -1 if missing or bad number typed
 */

int parse_num (ch, prompt)
	char ch;
	char *prompt;
{
	char buf[40];
	int len;
	int i;
	int num;
	int	ic = ch & 0xFF;

	MoveCursor (LINES,0);
	printf ("%s %c",prompt,ch);
	fflush (stdout);
	buf[0] = ch;
	buf[1] = '\0';
	len = 1;
 	ic = ReadCh ();
 	if(ic == EOF)
 		return (-1);
 	ic &= 0xFF;
 	while (ic != '\n' && ic != '\r') {
		if (ic == ESC) {		/* Esc pressed so return */
			clear_message ();
			return -1;
		}
 		if (ic == '\b' || ic == 127) {
			if (len) {
				len--;
				buf[len] = '\0';
				putchar ('\b');
				putchar (' ');
				putchar ('\b');
			} else {
				MoveCursor (LINES, 0);
				CleartoEOLN ();
				return (-1);
			}
		} else if (ic == 21) {	/* control-U	*/
			for (i = len; i > 0; i--) {
				putchar ('\b');
				putchar (' ');
				putchar ('\b');
			}
			buf[0] = '\0';
			len = 0;
		} else if (ic >= '0' && ic <= '9' && len < 4) {
			buf[len++] = ic;
			buf[len] = '\0';
			putchar (ic);
		} else
			putchar (7);
		fflush (stdout);
		ic = ReadCh ();
		if(ic == EOF)
			return (-1);
		ic &= 0xFF;
	}

	MoveCursor (LINES, 0);
	CleartoEOLN ();

	if (len) {
		num = atoi (buf);
		return (num);
	} else
		return (-1);
}

/*
 *  parse_string
 *  get a string from the user
 *  Return TRUE if a valid string was typed, FALSE otherwise
 */

int parse_string (prompt, buf)
	char *prompt;
	char *buf;
{
	int ch;
	int i, len;
	int max_len;

	clear_message ();
	MoveCursor (LINES,0);
	printf ("%s", prompt);
	fflush (stdout);

	buf[0] = '\0';
	len = 0;
	max_len = (COLS - strlen (prompt))-2;
	ch = ReadCh ();
 	if(ch == EOF)
 		return FALSE;
 	ch &= 0xFF;
	while (ch != '\n' && ch != '\r') {
		if (ch == ESC) {		/* Esc pressed so abort */
			clear_message ();
			return FALSE;
		}
		if (ch == '\b' || ch == 127) {
			if (len) {
				len--;
				buf[len] = '\0';
				putchar ('\b');
				putchar (' ');
				putchar ('\b');
			} else {
				MoveCursor (LINES, 0);
				CleartoEOLN ();
				return (FALSE);
			}
		} else if (ch == 21) {	/* control-U	*/
			for (i = len ; i > 0 ; i--) {
				putchar ('\b');
				putchar (' ');
				putchar ('\b');
			}
			buf[0] = '\0';
			len = 0;
		} else if (ch >= ' ' && len < max_len) {
			buf[len++] = ch;
			buf[len] = '\0';
			putchar (ch);
		} else {
			putchar (7);
		}
		fflush (stdout);
		ch = ReadCh ();
	}
	MoveCursor (LINES,0);
	CleartoEOLN ();

	return TRUE;
}

/*
 *  parse_menu_string
 *  get a string from the user
 *  Return TRUE if a valid string was typed, FALSE otherwise
 */

int parse_menu_string (line, col, var)
	int line;
	int col;
	char *var;
{
	char buf[LEN];
	char ch;
	int i, len;
	int max_len;

	MoveCursor (line, col);
	buf[0] = '\0';
	len = 0;
	max_len = (COLS - col)-2;
	ch = ReadCh();

	while (ch != '\n' && ch != '\r') {
		if (ch == ESC) {		/* Esc pressed so abort */
			return FALSE;
		}
		if (ch == 8 || ch == 127) {
			if (len) {
				len--;
				buf[len] = '\0';
				putchar('\b');
				putchar(' ');
				putchar('\b');
			} else {
				strcpy (var, buf);
				MoveCursor(line, col);
				CleartoEOLN();
			}
		} else if (ch == 21) {	/* control-U	*/
			for (i = len; i > 0; i--) {
				putchar('\b');
				putchar(' ');
				putchar('\b');
			}
			buf[0] = '\0';
			len = 0;
		} else if (ch >= ' ' && len < max_len) {
			buf[len++] = ch;
			buf[len] = '\0';
			putchar (ch);
		} else
			putchar(7);
		fflush(stdout);
		ch = ReadCh();
		if(ch == EOF)
			return FALSE;
		ch &= 0xFF;
	}

	if (buf[0]) {
		strcpy (var, buf);
	}
	return TRUE;
}


int prompt_yn (line, prompt, default_ch)
	int line;
	char *prompt;
	char default_ch;
{
	char ch;

	MoveCursor (line, 0);
	CleartoEOLN ();
	printf ("%s%c", prompt, default_ch);
	fflush (stdout);
	MoveCursor (line, (int) strlen (prompt));

	if ((ch = ReadCh()) == CR) {
		ch = default_ch;
	}	

	if (line == LINES) {
		clear_message();
	} else {
		MoveCursor (line, (int) strlen (prompt));
		if (ch == ESC) {
			printf ("%c", default_ch);
		} else {
			printf ("%c", ch);
		}
		fflush (stdout);
	}

	return (ch == 'y' ? TRUE : FALSE);
}


void continue_prompt()
{
	info_message(txt_hit_any_key);
	ReadCh();
}


