/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : rcfile.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 09-10-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

#define COL2	COLS/2

extern char index_file[LEN];

/*
 *  read_rcfile - read defaults from ~/.tin/tinrc
 */

int read_rcfile ()
{
	char buf[LEN];
	FILE *fp;

	if ((fp = fopen (rcfile, "r")) != NULL) {
		while (fgets (buf, sizeof buf, fp) != NULL) {
			if (buf[0] != '#') { 
				if (strncmp (buf, "save_archive=", 13) == 0) {
					save_archive_name = (strncmp (&buf[13], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "save_separate=", 14) == 0) {
					save_separate = (strncmp (&buf[14], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "mark_saved_read=", 16) == 0) {
					mark_saved_read = (strncmp (&buf[16], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "kill_articles=", 14) == 0) {
					kill_articles = (strncmp (&buf[14], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "show_author=", 12) == 0) {
					show_author = (strncmp (&buf[12], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "draw_arrow=", 11) == 0) {
					draw_arrow_mark = (strncmp (&buf[11], "ON", 2) == 0 ? TRUE : FALSE);
					if (draw_arrow_mark == FALSE && inverse_okay == FALSE) {
						inverse_okay = TRUE;
					}
				} else if (strncmp (buf, "print_header=", 13) == 0) {
					print_header = (strncmp (&buf[13], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "pos_first_unread=", 17) == 0) {
					pos_first_unread = (strncmp (&buf[17], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "thread_articles=", 16) == 0) {
					thread_arts = (strncmp (&buf[16], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "post_process_type=", 18) == 0) {
					post_proc_type = atoi (&buf[18]);
					switch (post_proc_type) {
						case POST_PROC_NONE:
							proc_ch_default = 'n';
							break;
						case POST_PROC_SHAR:
							proc_ch_default = 's';
							break;
						case POST_PROC_UUDECODE:
							proc_ch_default = 'u';
							break;
						case POST_PROC_UUD_LST_ZOO:
							proc_ch_default = 'U';
							break;
						case POST_PROC_UUD_EXT_ZOO:
							proc_ch_default = 'U';
							break;
						case POST_PROC_PATCH:
							proc_ch_default = 'p';
							break;
					}
				} else if (strncmp (buf, "sort_article_type=", 18) == 0) {
					sort_art_type = atoi (&buf[18]);
				} else if (strncmp (buf, "savedir=", 8) == 0) {
					strncpy (savedir, &buf[8], LEN);
					savedir[strlen (savedir) - 1] = '\0';
					if (savedir[0] == '.' && strlen (savedir) == 1) {
#if defined(BSD) && ! defined(SINIX)
						getwd (buf);	
#else
						getcwd (buf, LEN);
#endif
						my_strncpy (savedir, buf, LEN);
					}
				} else if (strncmp (buf, "maildir=", 8) == 0) {
					strncpy (maildir, &buf[8], LEN);
					maildir[strlen (maildir) - 1] = '\0';
				} else if (strncmp (buf, "printer=", 8) == 0) {
					strncpy (printer, &buf[8], LEN);
					printer[strlen (printer) - 1] = '\0';
				} else if (strncmp (buf, "spooldir=", 9) == 0) {
					strncpy (spooldir, &buf[9], LEN);
					spooldir[strlen (spooldir) - 1] = '\0';
				} else if (strncmp (buf, "signature=", 10) == 0) {
					strncpy (signature, &buf[10], LEN);
					signature[strlen (signature) - 1] = '\0';
				} else if (strncmp (buf, "sig=", 4) == 0) {
					strncpy (sig, &buf[4], LEN);
					sig[strlen (sig) - 1] = '\0';
				}
			}
		}
		fclose (fp);
		return TRUE;		
	}
	return FALSE;		
}

/*
 *  write_rcfile - write defaults to ~/.tin/tinrc
 */

void write_rcfile ()
{
	FILE *fp;

	set_real_uid_gid ();
	
	if ((fp = fopen (rcfile, "w")) != NULL) {
		wait_message (txt_saving);

		fprintf (fp, "# if ON articles/threads with Archive-name: in mail header will\n");
		fprintf (fp, "# be automatically saved with the Archive-name & part/patch no.\n");
		fprintf (fp, "save_archive=%s\n\n", (save_archive_name ? "ON" : "OFF"));
		fprintf (fp, "# if ON articles of a threads will be saved to separate files\n");
		fprintf (fp, "# otherwise the whole thread will be saved to one file\n");
		fprintf (fp, "save_separate=%s\n\n", (save_separate ? "ON" : "OFF"));
		fprintf (fp, "# if ON mark articles that are saved as read\n");
		fprintf (fp, "mark_saved_read=%s\n\n", (mark_saved_read ? "ON" : "OFF"));
		fprintf (fp, "# if ON show Subject & From otherwise just Subject\n");
		fprintf (fp, "show_author=%s\n\n", (show_author ? "ON" : "OFF"));
		fprintf (fp, "# if ON use -> otherwise highlighted bar for selection\n");
		fprintf (fp, "draw_arrow=%s\n\n", (draw_arrow_mark ? "ON" : "OFF"));
		fprintf (fp, "# if ON kill articles that match kill file\n");
		fprintf (fp, "kill_articles=%s\n\n", (kill_articles ? "ON" : "OFF"));
		fprintf (fp, "# if ON print all of mail header otherwise Subject: & From: lines\n");
		fprintf (fp, "print_header=%s\n\n", (print_header ? "ON" : "OFF"));
		fprintf (fp, "# if ON put cursor at first unread art in group otherwise last art\n");
		fprintf (fp, "pos_first_unread=%s\n\n", (pos_first_unread ? "ON" : "OFF"));
		fprintf (fp, "# type of post processing to perform after saving articles.\n");
		fprintf (fp, "# 0) none 1) shar 2) uudecode 3) uud & list zoo 4) uud & extract zoo 5) patch.\n");
		fprintf (fp, "post_process_type=%d\n\n", post_proc_type);
		fprintf (fp, "# if ON and group not in ~/.tin/unthread articles will be threaded.\n");
		fprintf (fp, "thread_articles=%s\n\n", (thread_arts ? "ON" : "OFF"));
		fprintf (fp, "# sort articles by 0) nothing 1) Subject (descending) 2) Subject (ascending)\n");
		fprintf (fp, "# 3) From (descend) 4) From (ascend) 5) Date (descend) 6) Date (ascend)\n");
		fprintf (fp, "sort_article_type=%d\n\n", sort_art_type);
		fprintf (fp, "# (-d) directory where articles/threads are saved\n");
		fprintf (fp, "savedir=%s\n\n", savedir);
		fprintf (fp, "# (-M) directory where articles/threads are saved in mailbox format\n");	
		fprintf (fp, "maildir=%s\n\n", maildir);	
		fprintf (fp, "# (-p) print program with parameters used to print articles/threads\n");
		fprintf (fp, "printer=%s\n\n", printer);
		fprintf (fp, "# (-s) directory where news is spooled\n");
		fprintf (fp, "spooldir=%s\n\n", spooldir);

		fprintf (fp, "# .signature file used for replies, followups\n");
		fprintf (fp, "signature=%s\n\n", signature);
		fprintf (fp, "# .Sig file used for postings\n");
		fprintf (fp, "sig=%s\n\n", sig);

		fclose (fp);
		chmod (rcfile, 0600);
	}
	set_tin_uid_gid ();
}

/*
 *  options menu so that the user can dynamically change parameters
 */
 
int change_rcfile (group, kill_at_once)
	char *group;
	int kill_at_once;
{
	char *str;
	int ch;
	int kill_changed = FALSE;
	int orig_kill_state;
	int option;
	int ret_code = NO_KILLING;
#ifdef SIGTSTP
	void (*susp)();

	if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
		sigemptyset (&rcfile_act.sa_mask);
		rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
		rcfile_act.sa_handler = SIG_DFL;
		sigaction (SIGTSTP, &rcfile_act, &old_act);
#else
		susp = signal (SIGTSTP, SIG_DFL);
#endif
	}
#endif

	show_rcfile_menu ();

	while (1) {

#ifdef SIGTSTP
		if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
			sigemptyset (&rcfile_act.sa_mask);
			rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
			rcfile_act.sa_handler = rcfile_suspend;
			sigaction (SIGTSTP, &rcfile_act, 0L);
#else
			signal (SIGTSTP, rcfile_suspend);
#endif
		}
#endif
		MoveCursor (LINES, 0);
		ch = ReadCh ();
		if (ch >= '1' && ch <= '9') {
			option = parse_num (ch, "Enter option number> ");
		} else {
			option = 0;
		}
#ifdef SIGTSTP
		if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
			sigemptyset (&rcfile_act.sa_mask);
			rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
			rcfile_act.sa_handler = SIG_IGN;
			sigaction (SIGTSTP, &rcfile_act, 0L);
#else
			signal (SIGTSTP, SIG_IGN);
#endif
		}
#endif
		switch (option) {
			case 0:
				write_rcfile ();
				if (kill_changed) {
					if (kill_at_once) {
						if (kill_articles) {
							read_kill_file ();
							if (kill_any_articles (group)) {
								reload_index_file (group, TRUE);	/* kill arts */
							}
						} else {
							reload_index_file (group, FALSE);	/* add killed arts */
						}
					}
					ret_code = KILLING;
				}
				clear_note_area ();
#ifdef SIGTSTP
				if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
					sigemptyset (&rcfile_act.sa_mask);
					rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
					rcfile_act.sa_handler = SIG_IGN;
					sigaction (SIGTSTP, &old_act, 0L);
#else
					signal (SIGTSTP, susp);
#endif
				}
#endif
				return ret_code;
			
			case 1:		/* auto save */
				show_menu_help (txt_help_autosave);
				do {
					MoveCursor (INDEX_TOP, (int) strlen (txt_opt_autosave));
					if ((ch = ReadCh()) == ' ') {
						save_archive_name = !save_archive_name;
						printf ("%s", (save_archive_name ? "ON " : "OFF"));
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;

			case 2:		/* save sperate */
				show_menu_help (txt_help_save_separate);
				do {
					MoveCursor (INDEX_TOP, COL2 + (int) strlen (txt_opt_save_separate));
					if ((ch = ReadCh()) == ' ') {
						save_separate = !save_separate;
						printf ("%s", (save_separate ? "ON " : "OFF"));
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;
			
			case 3:		/* mark saved articles read */
				show_menu_help (txt_help_mark_saved_read);
				do {
					MoveCursor (INDEX_TOP+2, (int) strlen (txt_opt_mark_saved_read));
					if ((ch = ReadCh()) == ' ') {
						mark_saved_read = !mark_saved_read;
						printf ("%s", (mark_saved_read ? "ON " : "OFF"));
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;

			case 4:		/* kill articles */
				orig_kill_state = kill_articles;
				show_menu_help (txt_help_kill_articles);
				do {
					MoveCursor (INDEX_TOP+2, COL2 + (int) strlen (txt_opt_kill_articles));
					if ((ch = ReadCh()) == ' ') {
						kill_articles = !kill_articles;
						kill_changed = (kill_articles != orig_kill_state ? TRUE : FALSE);
						printf ("%s", (kill_articles ? "ON " : "OFF"));
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;

			case 5:		/* show subject & author / subject only */
				show_menu_help (txt_help_show_author);
				do {
					MoveCursor (INDEX_TOP+4, (int) strlen (txt_opt_show_author));
					if ((ch = ReadCh()) == ' ') {
						show_author = !show_author;
						printf ("%s", (show_author ? "ON " : "OFF"));
						fflush(stdout);	
					}
				} while (ch != CR && ch != ESC);
				break;
			
			case 6:		/* draw -> / highlighted bar */
				show_menu_help (txt_help_draw_arrow);
				do {
					MoveCursor (INDEX_TOP+4, COL2 + (int) strlen (txt_opt_draw_arrow));
					if ((ch = ReadCh()) == ' ') {
						draw_arrow_mark = !draw_arrow_mark;
						printf ("%s", (draw_arrow_mark ? "ON " : "OFF"));
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				if (draw_arrow_mark == FALSE && inverse_okay == FALSE) {
					inverse_okay = TRUE;
				}
				break;

			case 7:		/* print header */
				show_menu_help (txt_help_print_header);
				do {
					MoveCursor (INDEX_TOP+6, (int) strlen (txt_opt_print_header));
					if ((ch = ReadCh()) == ' ') {
						print_header = !print_header;
						printf ("%s", (print_header ? "ON " : "OFF"));
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;
			
			case 8:		/* position cursor at first / last unread art */
				show_menu_help (txt_help_pos_first_unread);
				do {
					MoveCursor (INDEX_TOP+6, COL2 + (int) strlen (txt_opt_pos_first_unread));
					if ((ch = ReadCh()) == ' ') {
						pos_first_unread = !pos_first_unread;
						printf ("%s", (pos_first_unread ? "ON " : "OFF"));
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;

			case 9:
				show_menu_help (txt_help_post_proc_type);
				do {
					MoveCursor (INDEX_TOP+8, (int) strlen (txt_opt_process_type));
					if ((ch	= ReadCh()) == ' ') {
						if (post_proc_type + 1 > POST_PROC_PATCH) {
							post_proc_type = POST_PROC_NONE;
						} else {
							post_proc_type++;
						}
						switch (post_proc_type) {
							case POST_PROC_NONE:
								str = txt_post_process_none;
								proc_ch_default = 'n';
								break;
							case POST_PROC_SHAR:
								str = txt_post_process_sh;
								proc_ch_default = 's';
								break;
							case POST_PROC_UUDECODE:
								str = txt_post_process_uudecode;
								proc_ch_default = 'u';
								break;
							case POST_PROC_UUD_LST_ZOO:
								str = txt_post_process_uud_lst_zoo;
								proc_ch_default = 'U';
								break;
							case POST_PROC_UUD_EXT_ZOO:
								str = txt_post_process_uud_ext_zoo;
								proc_ch_default = 'U';
								break;
							case POST_PROC_PATCH:
								str = txt_post_process_patch;
								proc_ch_default = 'p';
								break;
						}
						CleartoEOLN (); 
						printf ("%s", str);
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;
			case 10:
				show_menu_help (txt_help_sort_type);
				do {
					MoveCursor (INDEX_TOP+10, (int) strlen (txt_opt_sort_type));
					if ((ch	= ReadCh()) == ' ') {
						if (sort_art_type + 1 > SORT_BY_DATE_ASCEND) {
							sort_art_type = SORT_BY_NOTHING;
						} else {
							sort_art_type++;
						}
						switch (sort_art_type) {
							case SORT_BY_NOTHING:
								str = txt_sort_by_nothing;
								break;
							case SORT_BY_SUBJ_DESCEND:
								str = txt_sort_by_subj_descend;
								break;
							case SORT_BY_SUBJ_ASCEND:
								str = txt_sort_by_subj_ascend;
								break;
							case SORT_BY_FROM_DESCEND:
								str = txt_sort_by_from_descend;
								break;
							case SORT_BY_FROM_ASCEND:
								str = txt_sort_by_from_ascend;
								break;
							case SORT_BY_DATE_DESCEND:
								str = txt_sort_by_date_descend;
								break;
							case SORT_BY_DATE_ASCEND:
								str = txt_sort_by_date_ascend;
								break;
						}
						CleartoEOLN (); 
						printf ("%s", str);
						fflush(stdout);
					}
				} while (ch != CR && ch != ESC);
				break;
			case 11:
				show_menu_help (txt_help_savedir);
				parse_menu_string (INDEX_TOP+12, (int) strlen (txt_opt_savedir), savedir);
				expand_rel_abs_pathname (INDEX_TOP+12, (int) strlen (txt_opt_savedir), savedir);
				break;
			case 12:
				show_menu_help (txt_help_maildir);
				parse_menu_string (INDEX_TOP+14, (int) strlen (txt_opt_maildir), maildir);
				expand_rel_abs_pathname (INDEX_TOP+14, (int) strlen (txt_opt_maildir), maildir);
				break;
			case 13:
				show_menu_help (txt_help_printer);
				parse_menu_string (INDEX_TOP+16, (int) strlen (txt_opt_printer), printer);
				expand_rel_abs_pathname (INDEX_TOP+16, (int) strlen (txt_opt_printer), printer);
				break;
		}
		show_menu_help (txt_select_rcfile_option);
	}
}


void show_rcfile_menu ()
{
	char *str;

	ClearScreen ();

	center_line (0, TRUE, txt_options_menu);
	
	MoveCursor (INDEX_TOP, 0);
	printf ("%s%s\r\n\r\n", txt_opt_autosave, (save_archive_name ? "ON " : "OFF"));
	printf ("%s%s\r\n\r\n", txt_opt_mark_saved_read, (mark_saved_read ? "ON " : "OFF"));
	printf ("%s%s\r\n\r\n", txt_opt_show_author, (show_author ? "ON " : "OFF"));
	printf ("%s%s", txt_opt_print_header, (print_header ? "ON " : "OFF"));

	MoveCursor(INDEX_TOP, COL2);
	printf ("%s%s", txt_opt_save_separate, (save_separate ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+2, COL2);
	printf ("%s%s", txt_opt_kill_articles, (kill_articles ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+4, COL2);
	printf ("%s%s", txt_opt_draw_arrow, (draw_arrow_mark ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+6, COL2);
	printf ("%s%s", txt_opt_pos_first_unread, (pos_first_unread ? "ON " : "OFF"));

	MoveCursor(INDEX_TOP+8, 0);
	switch (post_proc_type) {
		case POST_PROC_NONE:
			str = txt_post_process_none;
			break;
		case POST_PROC_SHAR:
			str = txt_post_process_sh;
			break;
		case POST_PROC_UUDECODE:
			str = txt_post_process_uudecode;
			break;
		case POST_PROC_UUD_LST_ZOO:
			str = txt_post_process_uud_lst_zoo;
			break;
		case POST_PROC_UUD_EXT_ZOO:
			str = txt_post_process_uud_ext_zoo;
			break;
		case POST_PROC_PATCH:
			str = txt_post_process_patch;
			break;
	}
	printf ("%s%s\r\n\r\n", txt_opt_process_type, str);
	
	MoveCursor(INDEX_TOP+10, 0);
	switch (sort_art_type) {
		case SORT_BY_NOTHING:
			str = txt_sort_by_nothing;
			break;
		case SORT_BY_SUBJ_DESCEND:
			str = txt_sort_by_subj_descend;
			break;
		case SORT_BY_SUBJ_ASCEND:
			str = txt_sort_by_subj_ascend;
			break;
		case SORT_BY_FROM_DESCEND:
			str = txt_sort_by_from_descend;
			break;
		case SORT_BY_FROM_ASCEND:
			str = txt_sort_by_from_ascend;
			break;
		case SORT_BY_DATE_DESCEND:
			str = txt_sort_by_date_descend;
			break;
		case SORT_BY_DATE_ASCEND:
			str = txt_sort_by_date_ascend;
			break;
	}
	printf ("%s%s\r\n\r\n", txt_opt_sort_type, str);

	printf ("%s%s\r\n\r\n", txt_opt_savedir, savedir);
	printf ("%s%s\r\n\r\n", txt_opt_maildir, maildir);
	printf ("%s%s\r\n\r\n", txt_opt_printer, printer);
	fflush(stdout);

	show_menu_help (txt_select_rcfile_option);
	MoveCursor (LINES, 0);
}

/*
 *  expand ~/News to /usr/username/News and print to screen
 */
 
void expand_rel_abs_pathname (line, col, str)
	int line;
	int col;
	char *str;
{
	char buf[LEN];
	
	if (str[0] == '~') {
		if (strlen (str) == 1) {
			strcpy (str, homedir);
		} else {
			sprintf (buf, "%s%s", homedir, str+1);
			strcpy (str, buf);
		}
	}
	MoveCursor (line, col);
	CleartoEOLN ();
	puts (str);
	fflush (stdout);
}

/*
 *  show_menu_help
 */
 
void show_menu_help (help_message)
	char *help_message;
{
	 MoveCursor (LINES-2, 0);
	 CleartoEOLN ();
	 center_line (LINES-2, FALSE, help_message);
}
