/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : save.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 10-10-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta, Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

#define	INITIAL		1
#define MIDDLE		2
#define OFF			3
#define END			4

int create_subdir = TRUE;

struct save_t *save;
int save_num=0;
int max_save;

/*
 * types of archive programs
 * 0=archiver, 1=extension, 2=extract option, 3=list option
 */
 
struct archiver_t { 
	char *name;
	char *ext;
	char *extract;
	char *list;
};

struct archiver_t archiver[] = {
	{ "",			"",			"",			"" },
	{ "",			"",			"",			"" },
	{ "",			"",			"",			"" },
	{ "zoo",		"zoo",		"-extract",	"-list" },
	{ (char *) 0,	(char *) 0,	(char *) 0,	(char *) 0 }
};

extern char *glob_group;
extern char note_h_path[LEN];	/* Path:	*/
extern char note_h_date[LEN];	/* Date:	*/
extern FILE	*note_fp;			/* the body of the current article */
extern int index_point;
extern int note_end;
extern int note_page;
extern long note_mark[MAX_PAGES];


int save_art_to_file (respnum, index, mailbox, filename)
	int respnum;
	int index;
	int mailbox;
	char *filename;
{
	char file[LEN];
	char save_art_info[LEN];
	FILE *fp;
	int is_mailbox = FALSE;
	int i = 0, ret_code = FALSE;
	long epoch;
	
	if (filename) {
		my_strncpy (file, filename, LEN);
		is_mailbox = mailbox;
		i = index;
	} else if (save_archive_name && arts[respnum].archive) {
			my_strncpy (file, arts[respnum].archive, LEN);
	}

	if (! append_to_existing_file (i)) {
		save[i].saved = FALSE;
		info_message (txt_art_not_saved);
		sleep (1);
		return (ret_code);
	}

	set_real_uid_gid ();

	if ((fp = fopen (save_filename (i), "a+")) == NULL) {
		save[i].saved = FALSE;
		info_message (txt_art_not_saved);
		set_tin_uid_gid ();
		return (ret_code);
	}

 	time (&epoch);
 	fprintf(fp, "From %s %s", note_h_path, ctime (&epoch));

	if (fseek(note_fp, 0L, 0) == -1) {
		error_message ("fseek() error on [%s]", arts[respnum].subject);
	}
	copy_fp(note_fp, fp, "");
	fputs("\n", fp);
	fclose(fp);
	fseek(note_fp, note_mark[note_page], 0);

	save[i].saved = TRUE;

	if (mark_saved_read) {
		arts[respnum].unread = ART_READ;
	}
	
	set_tin_uid_gid ();

	if (! filename) {
		if (is_mailbox) {
			sprintf (save_art_info, txt_saved_to_mailbox, get_first_savefile ());
		} else {
			sprintf (save_art_info, txt_art_saved_to, get_first_savefile ());
		}
		info_message(save_art_info);
	}
	return TRUE;
}


int save_thread_to_file (is_mailbox, group_path)
	int is_mailbox;
	char *group_path;
{
	char save_thread_info[LEN];
	char *first_savefile;
	FILE *fp;
	int count = 0;
	int i, ret_code = FALSE;
	long	epoch;

	set_real_uid_gid ();

	for (i=0 ; i < save_num ; i++) {
		if (! append_to_existing_file (i)) {
			save[i].saved = FALSE;
			info_message (txt_art_not_saved);
			sleep (1);
			continue;
		}
		sprintf (msg, "%s%d", txt_saving, ++count);
		wait_message (msg);

		if ((fp = fopen (save_filename (i), "a+")) == NULL) {
			info_message (txt_thread_not_saved);
			set_tin_uid_gid ();
			return (ret_code);
		}	

		art_open (arts[save[i].index].artnum, group_path);
		time (&epoch);
		fprintf (fp, "From %s %s", note_h_path, ctime (&epoch));
		fseek (note_fp, 0L, 0);
		copy_fp (note_fp, fp, "");
		art_close ();
		save[i].saved = TRUE;

		if (mark_saved_read) {
			arts[save[i].index].unread = ART_READ;
		}

		fclose (fp);
	}
	set_tin_uid_gid ();
	
	first_savefile = get_first_savefile ();

	if (first_savefile == (char *) 0) {
		info_message (txt_thread_not_saved);
	} else {
		if (is_mailbox) {
			sprintf (save_thread_info, txt_saved_to_mailbox, first_savefile);
		} else {
			if (save_num == 1) {
				sprintf (save_thread_info, txt_art_saved_to, first_savefile);
			} else {
				if (save_separate) {
					sprintf (save_thread_info, txt_thread_saved_to_many,
						first_savefile, get_last_savefile ());
				} else {
					sprintf (save_thread_info, txt_thread_saved_to,
						first_savefile);
				}
			}
			free (first_savefile);
			first_savefile = (char *) 0;
		}
		info_message (save_thread_info);
	}
	return TRUE;
}


int save_regex_arts (is_mailbox, group_path)
	int is_mailbox;
	char *group_path;
{
	char buf[LEN];
	char buf2[LEN];
	char mailbox[LEN];
	int i, ret_code; 	
	
	for (i=0 ; i < save_num ; i++) {
		sprintf(buf, "%s%d", txt_saving, i+1);
		wait_message (buf);

		if (is_mailbox) {
			strcpy (buf2, mailbox);
		}else {
			sprintf (buf2, "%s.%02d", save[i].file, i+1);
		}

		art_open (arts[save[i].index].artnum, group_path);
		ret_code = save_art_to_file (save[i].index, i, is_mailbox, buf2);
		art_close ();			
		save[i].saved = TRUE;
	}

	if (! save_num) {	
		info_message (txt_no_match);
	} else {
		if (is_mailbox) {
			sprintf (buf, txt_saved_to_mailbox, get_first_savefile ());
		} else {
			sprintf (buf,txt_saved_pattern_to,
				get_first_savefile (), get_last_savefile ());
		}
		info_message (buf);
	}
	return (ret_code);
}


int append_to_existing_file (i)
	int i;
{
	char buf[LEN];
	char *file;
	struct stat st;

	if (! save[i].is_mailbox && save_separate) {
		file = save_filename (i);
		if (stat(file, &st) != -1) {	
			sprintf (buf, txt_append_to_file, file); 
			if (! prompt_yn (LINES, buf, 'n')) {
				free (file);
				file = NULL;
				return FALSE;
			}
		}
		free (file);
		file = NULL;
	}
	
	return TRUE;
}


int create_path (path)
	char *path;
{
	char buf[LEN];
	char group[LEN];
	int i, j, len;
	struct stat st;
	
	/*
	 * save in mailbox format to ~/Mail/<group.name>
	 */
	if (path[0] == '=') {
		return TRUE;
	}

	/*
	 * if ~/file expand (ie. /usr/homedir/file)
	 */
	switch (path[0]) {
		case '~':
			my_strncpy (buf, path+1, LEN);
			sprintf (path, "%s%s", homedir, buf);
			break;
		case '+':
			my_strncpy (buf, path+1, LEN);
#ifdef USE_LONG_FILENAMES 
			strcpy (group, glob_group);
#else
			my_strncpy (group, glob_group, 14);
#endif
			/*
			 *  convert 1st letter to uppercase
			 */
			if (group[0] >= 'a' && group[0] <= 'z') {
				group[0] = group[0] - 32;
			}
			sprintf (path, "%s/%s/%s", savedir, group, buf);
			break;
		case '.':
			error_message ("Cannot create %s", buf);
			return FALSE;
			/* NOTREACHED */
		default:
			sprintf (buf, "%s/%s", savedir, path);
			my_strncpy (path, buf, LEN);
			break;
	}

	/*
	 *  create any directories, otherwise check
	 *  errno and give appropiate error message
	 */
	len = (int) strlen (path);
	
	for (i=0, j=0 ; i < len ; i++, j++) {
		buf[j] = path[i];
		if (i+1 < len && path[i+1] == '/') {
			buf[j+1] = '\0';
			if (stat (buf, &st) == -1) {
				if (mkdir (buf, 0755) == -1) {
					error_message ("Cannot create %s", buf);
					return FALSE;
				}
			}
		}
	}
	return FALSE;
}


int create_sub_dir (i)
	int i;
{
	char dir[LEN];
	struct stat st;

	if (! save[i].is_mailbox && save[i].archive) {
		sprintf (dir, "%s/%s", save[i].dir, save[i].archive);
		if (stat (dir, &st) == -1) {
			mkdir (dir, 0755);
			return TRUE;
		}
		if ((st.st_mode & S_IFMT) == S_IFDIR) {
			return TRUE;
		} else {
			return FALSE;
		}
	}
	return FALSE;
}

/*
 *  add files to be saved to save array
 */

void add_to_save_list (index, article, is_mailbox, path)
	int index;
	struct header *article;
	int is_mailbox;
	char *path;
{
	char dir[LEN];
	char file[LEN];
	int i;
	
	dir[0] = '\0';
	file[0] = '\0';

	if (save_num == max_save-1) {
		expand_save ();
	}

	save[save_num].index   = index;
	save[save_num].saved   = FALSE;
	save[save_num].is_mailbox = is_mailbox;
	save[save_num].dir     = (char *) 0;
	save[save_num].file    = (char *) 0;
	save[save_num].archive = (char *) 0;
	save[save_num].part    = (char *) 0;
	save[save_num].patch   = (char *) 0;

	save[save_num].subject = str_dup (article->subject);
	if (article->archive) {
		save[save_num].archive = str_dup (article->archive);
	}
	if (article->part) {
		save[save_num].part = str_dup (article->part);
	}
	if (article->patch) {
		save[save_num].patch = str_dup (article->patch);
	}

	if (is_mailbox) {
		if ((int) strlen (path) > 1) {
			if (path[0] == '=') {
				strcpy (file, path+1);
			} else {
				strcpy (file, path);
			}
		} else {
			strcpy (file, glob_group);
		}
		save[save_num].dir = maildir;
		save[save_num].file = str_dup (file);
	} else {
		if (path[0]) {
			for (i=strlen (path) ; i ; i--) {
				if (path[i] == '/') {
					strncpy (dir, path, i);
					dir[i] = '\0';
					strcpy (file, path+i+1);
					break;
				}
			}
		}
		
		if (dir[0]) {
			save[save_num].dir = str_dup (dir);
		} else {
			save[save_num].dir = str_dup (savedir);
		}

		if (file[0]) {
			save[save_num].file = str_dup (file);
		} else {
			if (path[0]) {
				save[save_num].file = str_dup (path);
			} else {
				save[save_num].file = save[save_num].archive;
			}
		}
	}
	save_num++;
}

/*
 *  print save array of files to be saved
 */

void sort_save_list ()
{
	qsort ((char *) save, save_num, sizeof (struct save_t), save_comp);
}

/*
 *  string comparison routine for the qsort()
 *  ie. qsort(array, 5, 32, save_comp);
 */

int save_comp (p1, p2)
	char *p1;
	char *p2;
{
	struct save_t *s1 = (struct save_t *)p1;
	struct save_t *s2 = (struct save_t *)p2;

	/* s1->subject less than s2->subject */
	if (strcmp (s1->subject, s2->subject) < 0) {
		return -1;
	}
	/* s1->subject greater than s2->subject */
	if (strcmp (s1->subject, s2->subject) > 0) {
		return 1;
	}
	return 0;
}


char *save_filename (i)
	int i;
{
	char *filename;

	filename = (char *) my_malloc (LEN);

	if (save[i].is_mailbox) {
		sprintf (filename, "%s/%s", save[i].dir, save[i].file);
		return (filename);
	}
	
	if (! save_separate || ! save_archive_name || (! save[i].part && ! save[i].patch)) {
		if (! save_separate || save_num == 1) {
			sprintf (filename, "%s/%s", save[i].dir, save[i].file);
		} else {
			sprintf (filename, "%s/%s.%02d", save[i].dir, save[i].file, i+1);
		}
	} else {
		if (save[i].part) {
			if (create_sub_dir (i)) {
				sprintf (filename, "%s/%s/%s.%s%s", save[i].dir, save[i].archive, save[i].archive, LONG_PATH_PART, save[i].part);
			} else {
				sprintf (filename, "%s/%s.%s%s", save[i].dir, save[i].archive, LONG_PATH_PART, save[i].part);
			}
		} else {
			if (save[i].patch) {
				if (create_sub_dir (i)) {
					sprintf (filename, "%s/%s/%s.%s%s", save[i].dir, save[i].archive, save[i].archive, LONG_PATH_PATCH, save[i].patch);
				} else {
					sprintf (filename, "%s/%s.%s%s", save[i].dir, save[i].archive, LONG_PATH_PATCH, save[i].patch);
				}
			} else {
	 	 		sprintf (filename, "%s/%s", save[i].dir, save[i].file);
			}
		}
	}

	return (filename);
}


char *get_first_savefile ()
{
	char *file;
	int i;

	for (i=0 ; i < save_num ; i++) {
		if (save[i].saved) {
			file = (char *) my_malloc (LEN);
			if (save[i].is_mailbox) {
				sprintf (file, "%s/%s", save[i].dir, save[i].file);
				return (file);
			} else {
				if (save[i].archive && save_archive_name) {
					if (save[i].part) {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PART, save[i].part);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PART, save[i].part);
						}
					} else {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PATCH, save[i].patch);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PATCH, save[i].patch);
						}
					}
				} else {
					if (! save_separate || save_num == 1) {
						sprintf (file, "%s", save[i].file);
					} else {
						sprintf (file, "%s.%02d", save[i].file, i+1);
					}
				}
				return (file);
			}
		}
	}
	return ((char *) 0);
}


char *get_last_savefile ()
{
	char *file;
	int i;
	
	for (i=save_num-1 ; i >= 0 ; i--) {
		if (save[i].saved) {
			file = (char *) my_malloc (LEN);
			if (save[i].is_mailbox) {
				sprintf (file, "%s/%s", save[i].dir, save[i].file);
				return (file);
			} else {
				if (save[i].archive && save_archive_name) {
					if (save[i].part) {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PART, save[i].part);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PART, save[i].part);
						}
					} else {
						if (create_subdir) {
							sprintf (file, "%s/%s.%s%s", save[i].archive, save[i].archive, LONG_PATH_PATCH, save[i].patch);
						} else {
							sprintf (file, "%s.%s%s", save[i].archive, LONG_PATH_PATCH, save[i].patch);
						}
					}
				} else {
					if (! save_separate || save_num == 1) {
						sprintf (file, "%s", save[i].file);
					} else {
						sprintf (file, "%s.%02d", save[i].file, i+1);
					}
				}
				return (file);
			}
		}
	}
	return ((char *) 0);
}


int post_process_files (proc_type_ch)
	char proc_type_ch;
{
	if (save_num) {
		wait_message (txt_post_processing);

		set_real_uid_gid();

		switch (proc_type_ch) {
			case 's':
				post_process_sh ();
				break;
				
			case 'u':
				post_process_uud (POST_PROC_UUDECODE);
				break;

			case 'U':
				if (post_proc_type == POST_PROC_UUD_EXT_ZOO) {
					post_process_uud (POST_PROC_UUD_EXT_ZOO);
				} else {
					post_process_uud (POST_PROC_UUD_LST_ZOO);
				}
				break;

			case 'p':
				post_process_patch ();
				break;
		}

		info_message (txt_post_processing_finished);
		sleep (1);
		return TRUE;
	}
	return FALSE;
}


void post_process_uud (pp)
	int pp;
{
	char s[LEN], t[LEN], u[LEN];
	char buf[LEN], *file;
	char file_out[LEN];
	char file_out_dir[LEN];
	FILE *fp_in, *fp_out;
	int i, state = INITIAL;
	int file_size = 0;
	struct stat st;
	
	t[0] = '\0';
	u[0] = '\0';

	my_strncpy (file_out_dir, save_filename (0), LEN);
	for (i=strlen(file_out_dir) ; i > 0 ; i--) {
		if (file_out_dir[i] == '/') {
			file_out_dir[i] = '\0';
			break;
		}
	}

	sprintf (file_out, "%s/tin.%05d", file_out_dir, getpid ());
	
	if ((fp_out = fopen (file_out, "a+")) == NULL) {
		error_message (txt_cannot_open, file_out);
	}


	for (i=0 ; i < save_num ; i++) {
		my_strncpy (buf, save_filename (i), LEN);

		if ((fp_in = fopen (buf, "r")) != NULL) {
			if (fgets (s, sizeof s, fp_in) == NULL) {
				fclose (fp_in);
				continue;
			}
			while (state != END) { 
				switch (state) {
					case INITIAL:
						if (! strncmp ("begin", s, 5)) {
							state = MIDDLE;
							fprintf (fp_out, "%s", s);
						}
						break;

					case MIDDLE:
						if (s[0] == 'M') {
							fprintf (fp_out, "%s", s);
						} else if (strncmp("end", s, 3)) {
							state = OFF;
						} else { /* end */
							state = END;
							if (u[0] != 'M') {
								fprintf (fp_out, "%s", u);
							}
							if (t[0] != 'M') {
							    fprintf (fp_out, "%s", t);
							}
							fprintf (fp_out, "%s\n", s);
						}
						break;

					case OFF:
						if ((s[0] == 'M') && (t[0] == 'M') && (u[0] == 'M')) {
							fprintf (fp_out, "%s", u);
							fprintf (fp_out, "%s", t);
							fprintf (fp_out, "%s", s);
							state = MIDDLE;
						} else if (! strncmp ("end", s, 3)) {
							state = END;
							if (u[0] != 'M') {
							    fprintf (fp_out, "%s", u);
							}
							if (t[0] != 'M') {
							    fprintf (fp_out, "%s", t);
							}
							fprintf (fp_out, "%s\n", s);
						}
						break;

					case END:
						break;

					default:
						fprintf (stderr, "\r\nerror: ASSERT - default state\n");
						fclose (fp_in);
						fclose (fp_out);
						unlink (file_out);
						return;
				}
				strcpy (u,t);
				strcpy (t,s);
				/*
				 *  read next line & if error goto next file in save array
				 */
				if (fgets (s, sizeof s, fp_in) == NULL) {
					break;
				}
			}
			fclose (fp_in);
		}
	}
	fclose (fp_out);

	/*
	 *  uudecode file
	 */
	wait_message (txt_uudecoding);
	
	sprintf (buf, "cd %s; uudecode %s", file_out_dir, file_out); 
	if (invoke_cmd (buf)) {
		set_real_uid_gid ();
		/*
		 *  sum file
		 */
		if ((file = get_archive_file (file_out_dir, "*")) != NULL) { 
			sprintf (buf, "%s %s", DEFAULT_SUM, file); 
			printf ("\r\n\r\nChecksum of %s...\r\n\r\n", file); 
			fflush (stdout);
			if ((fp_in = popen (buf, "r")) == NULL) {
				printf ("Cannot execute %s\r\n", buf); 
				fflush (stdout);
			} else {
				if (stat (file, &st) != -1) {
					file_size = (int) st.st_size;
				}
				if (fgets (buf, sizeof buf, fp_in) != NULL) {
					buf[strlen (buf)-1] = '\0';
				}
				fclose (fp_in);
				printf ("%s  %8d bytes\r\n", buf, file_size); 
				fflush (stdout);
			}
			free (file);
			file = (char *) 0;
		}
	}

	set_real_uid_gid ();

	if (pp > POST_PROC_UUDECODE) {
		sprintf (buf, "*.%s", archiver[pp].ext); 
		if ((file = get_archive_file (file_out_dir, buf)) != NULL) {
			if (pp == POST_PROC_UUD_EXT_ZOO) {
				sprintf (buf, "cd %s; %s %s %s", file_out_dir,
					archiver[pp].name, archiver[pp].extract, file);
				printf ("\r\n\r\nListing %s archive...\r\n", file); 
			} else {
				sprintf (buf, "cd %s; %s %s %s", file_out_dir,
					archiver[pp].name, archiver[pp].list, file);
				printf ("\r\n\r\nExtracting %s archive...\r\n", file);
			}
			fflush (stdout);
			free (file);
			file = (char *) 0;
			if (! invoke_cmd (buf)) {
				set_real_uid_gid ();
				error_message ("Post processing failed", "");
			}
			set_real_uid_gid ();
		}
	}
	delete_processed_files ();

	unlink (file_out);
}

/*
 *  Unpack /bin/sh archives
 */
 
void post_process_sh ()
{
	char buf[LEN];
	char file_in[LEN];
	char file_out[LEN];
	char file_out_dir[LEN];
	char *ptr1, *ptr2;
	FILE *fp_in, *fp_out;
	int found_header;
	int i, j;
	char sh_pattern_1[16];
	char sh_pattern_2[16];

	strcpy (sh_pattern_1, "#! /bin/sh");
	strcpy (sh_pattern_2, "#!/bin/sh");

	my_strncpy (file_out_dir, save_filename (0), LEN);
	for (i=strlen(file_out_dir) ; i > 0 ; i--) {
		if (file_out_dir[i] == '/') {
			file_out_dir[i] = '\0';
			break;
		}
	}

	sprintf (file_out, "%s/tin.%05d", file_out_dir, getpid ());

	for (j=0 ; j < save_num ; j++) {
		my_strncpy (file_in, save_filename (j), LEN);

		printf ("\r\nExtracting %s...\r\n", file_in);
		fflush (stdout);

		found_header = FALSE;
		
		if ((fp_out = fopen (file_out, "w")) != NULL) {
			if ((fp_in = fopen (file_in, "r")) != NULL) {
				while (! feof (fp_in)) {
					if (fgets (buf, sizeof buf, fp_in)) {
						/*
						 *  find #!/bin/sh or #! /bin/sh pattern
						 */
						if (!found_header) {
							ptr1 = sh_pattern_1;
							ptr2 = sh_pattern_2;
							if (str_str (buf, ptr1) != 0 ||
								str_str (buf, ptr2) != 0) {
								found_header = TRUE;
							}
						}
					
						/*
						 *  Write to temp file
						 */
						if (found_header) {
							fputs (buf, fp_out);
						}
					}
				}
				fclose (fp_in);
			}
			fclose (fp_out);

			sprintf (buf, "cd %s; sh %s", file_out_dir, file_out); 
			printf ("\r\n");
			fflush (stdout);
			Raw (FALSE);
			invoke_cmd (buf);
			set_real_uid_gid ();
			Raw (TRUE);
			unlink (file_out);
		}
	}
	delete_processed_files ();
}


char *get_archive_file (dir, ext)
	char *dir;
	char *ext;
{
	char buf[LEN];
	char *file = NULL;
	FILE *fp;
	
	sprintf (buf, "ls -t %s/%s", dir, ext);

	if ((fp = popen (buf, "r")) == NULL) {
		return (char *) 0;
	}

	if (fgets (buf, sizeof buf, fp) != NULL) {
		file = str_dup (buf);
		file[strlen (file)-1] = '\0';
	}
	
	fclose (fp);

	return (file);
}


void delete_processed_files ()
{
	int i;

	printf ("\r\n");
	fflush (stdout);
	
	if (prompt_yn (LINES, "Delete saved files that have been post processed? (y/n): ", 'y')) {
		wait_message (txt_deleting);

		for (i=0 ; i < save_num ; i++) {
			unlink (save_filename (i));
		}
	}
}


void post_process_patch ()
{
}
